@@ -1,3 +1,6 @@
+import junitbuild.extensions.requiredVersionFromLibs
+import junitbuild.extensions.dependencyFromLibs
+
 plugins {
 	id("me.champeau.jmh")
 }
@@ -11,10 +14,6 @@ dependencies {
 	jmhAnnotationProcessor(dependencyFromLibs("jmh-generator-annprocess"))
 }
 
-tasks.jmhJar {
-	notCompatibleWithConfigurationCache("https://github.com/melix/jmh-gradle-plugin/issues/229")
-}
-
 pluginManager.withPlugin("checkstyle") {
 	tasks.named<Checkstyle>("checkstyleJmh").configure {
 		// use same style rules as defined for tests
