@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -10,7 +10,7 @@
 
 package org.junit.platform.engine;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
+import static org.apiguardian.api.API.Status.MAINTAINED;
 import static org.apiguardian.api.API.Status.STABLE;
 
 import java.util.Optional;
@@ -33,15 +33,19 @@ public interface DiscoverySelector {
 	/**
 	 * Return the {@linkplain DiscoverySelectorIdentifier identifier} of this
 	 * selector.
-	 * <p>
-	 * The returned identifier has to be parsable by a corresponding
+	 *
+	 * <p>The returned identifier must be parsable by a corresponding
 	 * {@link DiscoverySelectorIdentifierParser}.
 	 *
-	 * @return the identifier of this selector or empty if it is not supported;
-	 * never {@code null}
+	 * <p>The default implementation returns {@link Optional#empty()}. Can be
+	 * overridden by concrete implementations.
+	 *
+	 * @return an {@link Optional} containing the identifier of this selector;
+	 * never {@code null} but potentially empty if the selector does not support
+	 * identifiers
 	 * @since 1.11
 	 */
-	@API(status = EXPERIMENTAL, since = "1.11")
+	@API(status = MAINTAINED, since = "1.13.3")
 	default Optional<DiscoverySelectorIdentifier> toIdentifier() {
 		return Optional.empty();
 	}
