@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -10,7 +10,9 @@
 
 package org.junit.platform.engine.discovery;
 
-import static org.apiguardian.api.API.Status.EXPERIMENTAL;
+import static java.util.stream.Collectors.toList;
+import static org.apiguardian.api.API.Status.DEPRECATED;
+import static org.apiguardian.api.API.Status.MAINTAINED;
 import static org.apiguardian.api.API.Status.STABLE;
 import static org.junit.platform.commons.util.CollectionUtils.toUnmodifiableList;
 
@@ -29,6 +31,8 @@ import java.util.stream.Stream;
 
 import org.apiguardian.api.API;
 import org.junit.platform.commons.PreconditionViolationException;
+import org.junit.platform.commons.io.Resource;
+import org.junit.platform.commons.support.ReflectionSupport;
 import org.junit.platform.commons.util.Preconditions;
 import org.junit.platform.commons.util.ReflectionUtils;
 import org.junit.platform.engine.DiscoverySelector;
@@ -281,6 +285,7 @@ public final class DiscoverySelectors {
 	 * @param classpathResourceName the name of the classpath resource; never
 	 * {@code null} or blank
 	 * @see #selectClasspathResource(String, FilePosition)
+	 * @see #selectClasspathResourceByName(Set)
 	 * @see ClasspathResourceSelector
 	 * @see ClassLoader#getResource(String)
 	 * @see ClassLoader#getResourceAsStream(String)
@@ -310,6 +315,7 @@ public final class DiscoverySelectors {
 	 * {@code null} or blank
 	 * @param position the position inside the classpath resource; may be {@code null}
 	 * @see #selectClasspathResource(String)
+	 * @see #selectClasspathResourceByName(Set)
 	 * @see ClasspathResourceSelector
 	 * @see ClassLoader#getResource(String)
 	 * @see ClassLoader#getResourceAsStream(String)
@@ -317,10 +323,81 @@ public final class DiscoverySelectors {
 	 */
 	public static ClasspathResourceSelector selectClasspathResource(String classpathResourceName,
 			FilePosition position) {
-		Preconditions.notBlank(classpathResourceName, "Classpath resource name must not be null or blank");
+		Preconditions.notBlank(classpathResourceName, "classpath resource name must not be null or blank");
 		return new ClasspathResourceSelector(classpathResourceName, position);
 	}
 
+	/**
+	 * Create a {@code ClasspathResourceSelector} for the supplied classpath
+	 * resources.
+	 *
+	 * <p>Since {@linkplain org.junit.platform.engine.TestEngine engines} are not
+	 * expected to modify the classpath, the supplied resource must be on the
+	 * classpath of the
+	 * {@linkplain Thread#getContextClassLoader() context class loader} of the
+	 * {@linkplain Thread thread} that uses the resulting selector.
+	 *
+	 * <p>Note: Since Java 9, all resources are on the module path. Either in
+	 * named or unnamed modules. These resources are also considered to be
+	 * classpath resources.
+	 *
+	 * @param classpathResources a set of classpath resources; never
+	 * {@code null} or empty. All resources must have the same name, may not
+	 * be {@code null} or blank.
+	 * @since 1.12
+	 * @see #selectClasspathResource(String, FilePosition)
+	 * @see #selectClasspathResource(String)
+	 * @see ClasspathResourceSelector
+	 * @see ReflectionSupport#tryToGetResources(String)
+	 * @deprecated Please use {@link #selectClasspathResourceByName(Set)} instead.
+	 */
+	@API(status = DEPRECATED, since = "1.14")
+	@Deprecated
+	public static ClasspathResourceSelector selectClasspathResource(
+			Set<org.junit.platform.commons.support.Resource> classpathResources) {
+		return selectClasspathResourceByName(classpathResources);
+	}
+
+	/**
+	 * Create a {@code ClasspathResourceSelector} for the supplied classpath
+	 * resources.
+	 *
+	 * <p>Since {@linkplain org.junit.platform.engine.TestEngine engines} are not
+	 * expected to modify the classpath, the supplied resource must be on the
+	 * classpath of the
+	 * {@linkplain Thread#getContextClassLoader() context class loader} of the
+	 * {@linkplain Thread thread} that uses the resulting selector.
+	 *
+	 * <p>Note: Since Java 9, all resources are on the module path. Either in
+	 * named or unnamed modules. These resources are also considered to be
+	 * classpath resources.
+	 *
+	 * <p>The {@link Set} supplied to this method should have a reliable iteration
+	 * order to support reliable discovery and execution order. It is therefore
+	 * recommended that the set be a {@link java.util.SequencedSet} (on Java 21
+	 * or higher), {@link java.util.SortedSet}, {@link java.util.LinkedHashSet},
+	 * or similar. Note that {@link Set#of(Object[])} and related {@code Set.of()}
+	 * methods do not guarantee a reliable iteration order.
+	 *
+	 * @param classpathResources a set of classpath resources; never
+	 * {@code null} or empty. All resources must have the same name, may not
+	 * be {@code null} or blank.
+	 * @since 1.14
+	 * @see #selectClasspathResource(String, FilePosition)
+	 * @see #selectClasspathResource(String)
+	 * @see ClasspathResourceSelector
+	 * @see org.junit.platform.commons.support.ResourceSupport#tryToGetResources(String)
+	 */
+	@API(status = MAINTAINED, since = "1.14")
+	public static ClasspathResourceSelector selectClasspathResourceByName(Set<? extends Resource> classpathResources) {
+		Preconditions.notEmpty(classpathResources, "classpath resources must not be null or empty");
+		Preconditions.containsNoNullElements(classpathResources, "individual classpath resources must not be null");
+		List<String> resourceNames = classpathResources.stream().map(Resource::getName).distinct().collect(toList());
+		Preconditions.condition(resourceNames.size() == 1, "all classpath resources must have the same name");
+		Preconditions.notBlank(resourceNames.get(0), "classpath resource names must not be null or blank");
+		return new ClasspathResourceSelector(classpathResources);
+	}
+
 	/**
 	 * Create a {@code ModuleSelector} for the supplied module name.
 	 *
@@ -407,7 +484,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see ClassSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static ClassSelector selectClass(ClassLoader classLoader, String className) {
 		Preconditions.notBlank(className, "Class name must not be null or blank");
 		return new ClassSelector(classLoader, className);
@@ -475,7 +552,7 @@ public final class DiscoverySelectors {
 	 * @see #selectMethod(String)
 	 * @see MethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static MethodSelector selectMethod(ClassLoader classLoader, String fullyQualifiedMethodName)
 			throws PreconditionViolationException {
 		String[] methodParts = ReflectionUtils.parseFullyQualifiedMethodName(fullyQualifiedMethodName);
@@ -507,7 +584,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see MethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static MethodSelector selectMethod(ClassLoader classLoader, String className, String methodName) {
 		return selectMethod(classLoader, className, methodName, "");
 	}
@@ -551,7 +628,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see MethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static MethodSelector selectMethod(ClassLoader classLoader, String className, String methodName,
 			String parameterTypeNames) {
 		Preconditions.notBlank(className, "Class name must not be null or blank");
@@ -607,7 +684,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see MethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static MethodSelector selectMethod(String className, String methodName, Class<?>... parameterTypes) {
 		Preconditions.notBlank(className, "Class name must not be null or blank");
 		Preconditions.notBlank(methodName, "Method name must not be null or blank");
@@ -628,7 +705,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see MethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static MethodSelector selectMethod(Class<?> javaClass, String methodName, Class<?>... parameterTypes) {
 		Preconditions.notNull(javaClass, "Class must not be null");
 		Preconditions.notBlank(methodName, "Method name must not be null or blank");
@@ -692,7 +769,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see NestedClassSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static NestedClassSelector selectNestedClass(ClassLoader classLoader, List<String> enclosingClassNames,
 			String nestedClassName) {
 		Preconditions.notEmpty(enclosingClassNames, "Enclosing class names must not be null or empty");
@@ -728,7 +805,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see NestedMethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static NestedMethodSelector selectNestedMethod(ClassLoader classLoader, List<String> enclosingClassNames,
 			String nestedClassName, String methodName) throws PreconditionViolationException {
 		Preconditions.notEmpty(enclosingClassNames, "Enclosing class names must not be null or empty");
@@ -776,7 +853,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see #selectNestedMethod(List, String, String, String)
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static NestedMethodSelector selectNestedMethod(ClassLoader classLoader, List<String> enclosingClassNames,
 			String nestedClassName, String methodName, String parameterTypeNames) {
 
@@ -802,7 +879,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see NestedMethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static NestedMethodSelector selectNestedMethod(List<String> enclosingClassNames, String nestedClassName,
 			String methodName, Class<?>... parameterTypes) {
 
@@ -871,7 +948,7 @@ public final class DiscoverySelectors {
 	 * @since 1.10
 	 * @see NestedMethodSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.10")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static NestedMethodSelector selectNestedMethod(List<Class<?>> enclosingClasses, Class<?> nestedClass,
 			String methodName, Class<?>... parameterTypes) {
 
@@ -930,12 +1007,12 @@ public final class DiscoverySelectors {
 	 *
 	 * @param parentSelector the parent selector to select iterations for; never
 	 * {@code null}
-	 * @param iterationIndices the iteration indices to select; never
-	 * {@code null} or empty
+	 * @param iterationIndices the iteration indices to select; never {@code null}
+	 * or empty
 	 * @since 1.9
 	 * @see IterationSelector
 	 */
-	@API(status = EXPERIMENTAL, since = "1.9")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static IterationSelector selectIteration(DiscoverySelector parentSelector, int... iterationIndices) {
 		Preconditions.notNull(parentSelector, "Parent selector must not be null");
 		Preconditions.notEmpty(iterationIndices, "iteration indices must not be empty");
@@ -943,15 +1020,16 @@ public final class DiscoverySelectors {
 	}
 
 	/**
-	 * Parse the supplied string representation of a
-	 * {@link DiscoverySelectorIdentifier}.
+	 * Parse the supplied string representation of a {@link DiscoverySelectorIdentifier}.
 	 *
 	 * @param identifier the string representation of a {@code DiscoverySelectorIdentifier};
 	 * never {@code null} or blank
+	 * @return an {@link Optional} containing the corresponding {@link DiscoverySelector};
+	 * never {@code null} but potentially empty
 	 * @since 1.11
 	 * @see DiscoverySelectorIdentifierParser
 	 */
-	@API(status = EXPERIMENTAL, since = "1.11")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static Optional<? extends DiscoverySelector> parse(String identifier) {
 		return DiscoverySelectorIdentifierParsers.parse(identifier);
 	}
@@ -961,10 +1039,12 @@ public final class DiscoverySelectors {
 	 *
 	 * @param identifier the {@code DiscoverySelectorIdentifier} to parse;
 	 * never {@code null}
+	 * @return an {@link Optional} containing the corresponding {@link DiscoverySelector};
+	 * never {@code null} but potentially empty
 	 * @since 1.11
 	 * @see DiscoverySelectorIdentifierParser
 	 */
-	@API(status = EXPERIMENTAL, since = "1.11")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static Optional<? extends DiscoverySelector> parse(DiscoverySelectorIdentifier identifier) {
 		return DiscoverySelectorIdentifierParsers.parse(identifier);
 	}
@@ -975,10 +1055,12 @@ public final class DiscoverySelectors {
 	 *
 	 * @param identifiers the string representations of
 	 * {@code DiscoverySelectorIdentifiers} to parse; never {@code null}
+	 * @return a stream of the corresponding {@link DiscoverySelector DiscoverySelectors};
+	 * never {@code null} but potentially empty
 	 * @since 1.11
 	 * @see DiscoverySelectorIdentifierParser
 	 */
-	@API(status = EXPERIMENTAL, since = "1.11")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static Stream<? extends DiscoverySelector> parseAll(String... identifiers) {
 		return DiscoverySelectorIdentifierParsers.parseAll(identifiers);
 	}
@@ -989,11 +1071,14 @@ public final class DiscoverySelectors {
 	 *
 	 * @param identifiers the {@code DiscoverySelectorIdentifiers} to parse;
 	 * never {@code null}
+	 * @return a stream of the corresponding {@link DiscoverySelector DiscoverySelectors};
+	 * never {@code null} but potentially empty
 	 * @since 1.11
 	 * @see DiscoverySelectorIdentifierParser
 	 */
-	@API(status = EXPERIMENTAL, since = "1.11")
+	@API(status = MAINTAINED, since = "1.13.3")
 	public static Stream<? extends DiscoverySelector> parseAll(Collection<DiscoverySelectorIdentifier> identifiers) {
 		return DiscoverySelectorIdentifierParsers.parseAll(identifiers);
 	}
+
 }
