@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -10,9 +10,12 @@
 
 package org.junit.platform.launcher;
 
+import static org.apiguardian.api.API.Status.MAINTAINED;
 import static org.apiguardian.api.API.Status.STABLE;
 
 import org.apiguardian.api.API;
+import org.junit.platform.engine.support.store.Namespace;
+import org.junit.platform.engine.support.store.NamespacedHierarchicalStore;
 import org.junit.platform.launcher.core.LauncherFactory;
 
 /**
@@ -47,4 +50,19 @@ public interface LauncherSession extends AutoCloseable {
 	@Override
 	void close();
 
+	/**
+	 * Get the {@link NamespacedHierarchicalStore} associated with this session.
+	 *
+	 * <p>All stored values that implement {@link AutoCloseable} are notified by
+	 * invoking their {@code close()} methods when this session is closed.
+	 *
+	 * <p>Any call to the store returned by this method after the session has
+	 * been closed will throw an exception.
+	 *
+	 * @since 1.13
+	 * @see NamespacedHierarchicalStore
+	 */
+	@API(status = MAINTAINED, since = "1.13.3")
+	NamespacedHierarchicalStore<Namespace> getStore();
+
 }
