@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2021 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -8,6 +8,7 @@
  * https://www.eclipse.org/legal/epl-v20.html
  */
 
+
 /**
  * Defines the JUnit Platform Reporting API.
  *
@@ -19,6 +20,7 @@ module org.junit.platform.reporting {
 	requires org.junit.platform.commons;
 	requires transitive org.junit.platform.engine;
 	requires transitive org.junit.platform.launcher;
+	requires org.opentest4j.reporting.tooling.spi;
 
 	// exports org.junit.platform.reporting; empty package
 	exports org.junit.platform.reporting.legacy;
@@ -27,4 +29,7 @@ module org.junit.platform.reporting {
 
 	provides org.junit.platform.launcher.TestExecutionListener
 			with org.junit.platform.reporting.open.xml.OpenTestReportGeneratingListener;
+
+	provides org.opentest4j.reporting.tooling.spi.htmlreport.Contributor
+			with org.junit.platform.reporting.open.xml.JUnitContributor;
 }
