@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -42,6 +42,7 @@ import org.junit.platform.engine.TestDescriptor;
 import org.junit.platform.engine.TestExecutionResult;
 import org.junit.platform.engine.TestExecutionResult.Status;
 import org.junit.platform.engine.UniqueId;
+import org.junit.platform.engine.reporting.FileEntry;
 import org.junit.platform.engine.reporting.ReportEntry;
 import org.junit.platform.engine.support.descriptor.EngineDescriptor;
 
@@ -247,6 +248,32 @@ public final class EventConditions {
 		return allOf(type(DYNAMIC_TEST_REGISTERED), condition);
 	}
 
+	/**
+	 * Create a new {@link Condition} that matches if and only if the
+	 * {@linkplain TestDescriptor#getUniqueId() unique id} of an {@link Event}'s
+	 * {@linkplain Event#getTestDescriptor() test descriptor} is equal to the
+	 * {@link UniqueId} parsed from the supplied {@link String}.
+	 *
+	 * @since 1.13
+	 */
+	@API(status = MAINTAINED, since = "1.13.3")
+	public static Condition<Event> uniqueId(String uniqueId) {
+		return uniqueId(UniqueId.parse(uniqueId));
+	}
+
+	/**
+	 * Create a new {@link Condition} that matches if and only if the
+	 * {@linkplain TestDescriptor#getUniqueId() unique id} of an {@link Event}'s
+	 * {@linkplain Event#getTestDescriptor() test descriptor} is equal to the
+	 * supplied {@link UniqueId}.
+	 *
+	 * @since 1.13
+	 */
+	@API(status = MAINTAINED, since = "1.13.3")
+	public static Condition<Event> uniqueId(UniqueId uniqueId) {
+		return uniqueId(new Condition<>(isEqual(uniqueId), "equal to '%s'", uniqueId));
+	}
+
 	/**
 	 * Create a new {@link Condition} that matches if and only if the
 	 * {@linkplain TestDescriptor#getUniqueId() unique id} of an
@@ -258,11 +285,22 @@ public final class EventConditions {
 			String text = segment.getType() + ":" + segment.getValue();
 			return text.contains(uniqueIdSubstring);
 		};
+		return uniqueId(new Condition<>(uniqueId -> uniqueId.getSegments().stream().anyMatch(predicate),
+			"substring '%s'", uniqueIdSubstring));
+	}
 
-		return new Condition<>(
-			byTestDescriptor(
-				where(TestDescriptor::getUniqueId, uniqueId -> uniqueId.getSegments().stream().anyMatch(predicate))),
-			"descriptor with uniqueId substring '%s'", uniqueIdSubstring);
+	/**
+	 * Create a new {@link Condition} that matches if and only if the
+	 * {@linkplain TestDescriptor#getUniqueId() unique id} of an {@link Event}'s
+	 * {@linkplain Event#getTestDescriptor() test descriptor} matches the
+	 * supplied {@link Condition}.
+	 *
+	 * @since 1.13
+	 */
+	@API(status = MAINTAINED, since = "1.13.3")
+	public static Condition<Event> uniqueId(Condition<? super UniqueId> condition) {
+		return new Condition<>(byTestDescriptor(where(TestDescriptor::getUniqueId, condition::matches)),
+			"descriptor with uniqueId %s", condition.description().value());
 	}
 
 	/**
@@ -313,6 +351,21 @@ public final class EventConditions {
 			"descriptor with display name '%s'", displayName);
 	}
 
+	/**
+	 * Create a new {@link Condition} that matches if and only if the
+	 * {@linkplain TestDescriptor#getLegacyReportingName()} () legacy reporting name}
+	 * of an {@link Event}'s {@linkplain Event#getTestDescriptor() test descriptor}
+	 * is equal to the supplied {@link String}.
+	 *
+	 * @since 1.13
+	 */
+	@API(status = MAINTAINED, since = "1.13.3")
+	public static Condition<Event> legacyReportingName(String legacyReportingName) {
+		return new Condition<>(
+			byTestDescriptor(where(TestDescriptor::getLegacyReportingName, isEqual(legacyReportingName))),
+			"descriptor with legacy reporting name '%s'", legacyReportingName);
+	}
+
 	/**
 	 * Create a new {@link Condition} that matches if and only if an
 	 * {@link Event}'s {@linkplain Event#getType() type} is
@@ -470,4 +523,17 @@ public final class EventConditions {
 			"event for report entry with key-value pairs %s", keyValuePairs);
 	}
 
+	/**
+	 * Create a new {@link Condition} that matches if and only if an
+	 * {@link Event}'s {@linkplain Event#getPayload() payload} is an instance of
+	 * {@link FileEntry} that contains a file that matches the supplied
+	 * {@link Predicate}.
+	 *
+	 * @since 1.12
+	 */
+	@API(status = MAINTAINED, since = "1.13.3")
+	public static Condition<Event> fileEntry(Predicate<FileEntry> predicate) {
+		return new Condition<>(byPayload(FileEntry.class, predicate), "event for file entry with custom predicate");
+	}
+
 }
