@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -31,8 +31,8 @@ class InterceptingExecutableInvokerTests extends AbstractExecutableInvokerTests
 
 	@Override
 	<T> T invokeConstructor(Constructor<T> constructor, Object outerInstance) {
-		return newInvoker().invoke(constructor, Optional.ofNullable(outerInstance), extensionContext, extensionRegistry,
-			passthroughInterceptor());
+		return newInvoker().invoke(constructor, Optional.ofNullable(outerInstance), __ -> extensionContext,
+			extensionRegistry, passthroughInterceptor());
 	}
 
 	private InterceptingExecutableInvoker newInvoker() {
