@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -14,6 +14,8 @@ import static org.assertj.core.api.Assertions.assertThat;
 
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.TestInfo;
+import org.junit.jupiter.api.TestReporter;
 
 /**
  * @since 5.9
@@ -28,7 +30,8 @@ public class SuperClassWithPackagePrivateLifecycleMethodInDifferentPackageTestCa
 	}
 
 	@Test
-	void test() {
+	void test(TestInfo testInfo, TestReporter reporter) {
+		reporter.publishEntry("invokedSuper", testInfo.getTestMethod().orElseThrow().toGenericString());
 		assertThat(this.beforeEachInvoked).isTrue();
 	}
 
