@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -14,7 +14,6 @@ import static java.lang.annotation.RetentionPolicy.RUNTIME;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.within;
 import static org.junit.jupiter.api.Assertions.assertEquals;
-import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
@@ -23,8 +22,8 @@ import static org.junit.jupiter.api.Named.named;
 import static org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS;
 import static org.junit.jupiter.engine.discovery.JupiterUniqueIdBuilder.appendTestTemplateInvocationSegment;
 import static org.junit.jupiter.engine.discovery.JupiterUniqueIdBuilder.uniqueIdForTestTemplateMethod;
+import static org.junit.jupiter.params.converter.DefaultArgumentConverter.DEFAULT_LOCALE_CONVERSION_FORMAT_PROPERTY_NAME;
 import static org.junit.jupiter.params.provider.Arguments.arguments;
-import static org.junit.platform.engine.discovery.DiscoverySelectors.selectClass;
 import static org.junit.platform.engine.discovery.DiscoverySelectors.selectIteration;
 import static org.junit.platform.engine.discovery.DiscoverySelectors.selectMethod;
 import static org.junit.platform.engine.discovery.DiscoverySelectors.selectUniqueId;
@@ -42,6 +41,7 @@ import static org.junit.platform.testkit.engine.TestExecutionResultConditions.me
 import java.lang.annotation.ElementType;
 import java.lang.annotation.Retention;
 import java.lang.annotation.Target;
+import java.lang.reflect.Constructor;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -81,15 +81,20 @@ import org.junit.jupiter.api.TestInstance;
 import org.junit.jupiter.api.TestInstance.Lifecycle;
 import org.junit.jupiter.api.TestMethodOrder;
 import org.junit.jupiter.api.TestReporter;
+import org.junit.jupiter.api.extension.AnnotatedElementContext;
 import org.junit.jupiter.api.extension.ExtensionContext;
 import org.junit.jupiter.api.extension.ParameterContext;
 import org.junit.jupiter.api.extension.ParameterResolutionException;
+import org.junit.jupiter.api.extension.ParameterResolver;
+import org.junit.jupiter.api.extension.RegisterExtension;
+import org.junit.jupiter.api.extension.TemplateInvocationValidationException;
+import org.junit.jupiter.engine.AbstractJupiterTestEngineTests;
 import org.junit.jupiter.engine.JupiterTestEngine;
 import org.junit.jupiter.params.ParameterizedTestIntegrationTests.RepeatableSourcesTestCase.Action;
 import org.junit.jupiter.params.aggregator.AggregateWith;
 import org.junit.jupiter.params.aggregator.ArgumentsAccessor;
 import org.junit.jupiter.params.aggregator.ArgumentsAggregationException;
-import org.junit.jupiter.params.aggregator.ArgumentsAggregator;
+import org.junit.jupiter.params.aggregator.SimpleArgumentsAggregator;
 import org.junit.jupiter.params.converter.ArgumentConversionException;
 import org.junit.jupiter.params.converter.ArgumentConverter;
 import org.junit.jupiter.params.converter.ConvertWith;
@@ -105,25 +110,28 @@ import org.junit.jupiter.params.provider.MethodSource;
 import org.junit.jupiter.params.provider.NullAndEmptySource;
 import org.junit.jupiter.params.provider.NullSource;
 import org.junit.jupiter.params.provider.ValueSource;
+import org.junit.jupiter.params.support.ParameterDeclarations;
 import org.junit.platform.commons.PreconditionViolationException;
 import org.junit.platform.commons.util.ClassUtils;
-import org.junit.platform.engine.DiscoverySelector;
 import org.junit.platform.engine.TestDescriptor;
+import org.junit.platform.engine.TestExecutionResult;
 import org.junit.platform.testkit.engine.EngineExecutionResults;
 import org.junit.platform.testkit.engine.EngineTestKit;
 import org.junit.platform.testkit.engine.Event;
+import org.junit.platform.testkit.engine.EventConditions;
 import org.opentest4j.TestAbortedException;
 
 /**
  * @since 5.0
  */
-class ParameterizedTestIntegrationTests {
+class ParameterizedTestIntegrationTests extends AbstractJupiterTestEngineTests {
 
 	private final Locale originalLocale = Locale.getDefault(Locale.Category.FORMAT);
 
 	@AfterEach
-	void restoreLocale() {
+	void reset() {
 		Locale.setDefault(Locale.Category.FORMAT, originalLocale);
+		AutoCloseableArgument.closeCounter = 0;
 	}
 
 	@ParameterizedTest
@@ -388,7 +396,7 @@ class ParameterizedTestIntegrationTests {
 		LifecycleTestCase.lifecycleEvents.clear();
 		LifecycleTestCase.testMethods.clear();
 
-		var results = execute(selectClass(LifecycleTestCase.class));
+		var results = executeTestsForClass(LifecycleTestCase.class);
 		results.allEvents().assertThatEvents() //
 				.haveExactly(1,
 					event(test("test1"), displayName("[1] argument=foo"), finishedWithFailure(message("foo")))) //
@@ -401,20 +409,20 @@ class ParameterizedTestIntegrationTests {
 		assertThat(LifecycleTestCase.lifecycleEvents).containsExactly(
 			"beforeAll:ParameterizedTestIntegrationTests$LifecycleTestCase",
 				"providerMethod",
-					"constructor:ParameterizedTestIntegrationTests$LifecycleTestCase",
+				"constructor:[1] argument=foo",
 					"beforeEach:[1] argument=foo",
 						testMethods.get(0) + ":[1] argument=foo",
 					"afterEach:[1] argument=foo",
-					"constructor:ParameterizedTestIntegrationTests$LifecycleTestCase",
+					"constructor:[2] argument=bar",
 					"beforeEach:[2] argument=bar",
 						testMethods.get(0) + ":[2] argument=bar",
 					"afterEach:[2] argument=bar",
 				"providerMethod",
-					"constructor:ParameterizedTestIntegrationTests$LifecycleTestCase",
+					"constructor:[1] argument=foo",
 					"beforeEach:[1] argument=foo",
 						testMethods.get(1) + ":[1] argument=foo",
 					"afterEach:[1] argument=foo",
-					"constructor:ParameterizedTestIntegrationTests$LifecycleTestCase",
+					"constructor:[2] argument=bar",
 					"beforeEach:[2] argument=bar",
 						testMethods.get(1) + ":[2] argument=bar",
 					"afterEach:[2] argument=bar",
@@ -425,7 +433,7 @@ class ParameterizedTestIntegrationTests {
 	@Test
 	void truncatesArgumentsThatExceedMaxLength() {
 		var results = EngineTestKit.engine(new JupiterTestEngine()) //
-				.configurationParameter(ParameterizedTestExtension.ARGUMENT_MAX_LENGTH_KEY, "2") //
+				.configurationParameter(ParameterizedInvocationNameFormatter.ARGUMENT_MAX_LENGTH_KEY, "2") //
 				.selectors(selectMethod(TestCase.class, "testWithCsvSource", String.class.getName())) //
 				.execute();
 		results.testEvents().assertThatEvents() //
@@ -436,7 +444,7 @@ class ParameterizedTestIntegrationTests {
 	@Test
 	void displayNamePatternFromConfiguration() {
 		var results = EngineTestKit.engine(new JupiterTestEngine()) //
-				.configurationParameter(ParameterizedTestExtension.DISPLAY_NAME_PATTERN_KEY, "{index}") //
+				.configurationParameter(ParameterizedInvocationNameFormatter.DISPLAY_NAME_PATTERN_KEY, "{index}") //
 				.selectors(selectMethod(TestCase.class, "testWithCsvSource", String.class.getName())) //
 				.execute();
 		results.testEvents().assertThatEvents() //
@@ -444,18 +452,88 @@ class ParameterizedTestIntegrationTests {
 				.haveExactly(1, event(displayName("2"), started()));
 	}
 
-	private EngineExecutionResults execute(DiscoverySelector... selectors) {
-		return EngineTestKit.engine(new JupiterTestEngine()).selectors(selectors).execute();
+	@Test
+	void failsWhenInvocationIsRequiredButNoArgumentSetsAreProvided() {
+		var results = execute(ZeroInvocationsTestCase.class, "testThatRequiresInvocations", String.class);
+
+		results.containerEvents().assertThatEvents() //
+				.haveExactly(1,
+					event(finishedWithFailure(instanceOf(TemplateInvocationValidationException.class), message(
+						"Configuration error: You must configure at least one set of arguments for this @ParameterizedTest"))));
 	}
 
-	private EngineExecutionResults execute(Class<?> testClass, String methodName, Class<?>... methodParameterTypes) {
-		return execute(selectMethod(testClass, methodName, ClassUtils.nullSafeToString(methodParameterTypes)));
+	@Test
+	void doesNotFailWhenInvocationIsNotRequiredAndNoArgumentSetsAreProvided() {
+		var results = execute(ZeroInvocationsTestCase.class, "testThatDoesNotRequireInvocations", String.class);
+
+		results.allEvents().assertStatistics(stats -> stats.started(3).succeeded(3));
+	}
+
+	@Test
+	void failsWhenNoArgumentsSourceIsDeclared() {
+		var results = execute(ZeroInvocationsTestCase.class, "testThatHasNoArgumentsSource", String.class);
+
+		results.containerEvents().assertThatEvents() //
+				.haveExactly(1, //
+					event(displayName("testThatHasNoArgumentsSource(String)"), finishedWithFailure(message(
+						"Configuration error: You must configure at least one arguments source for this @ParameterizedTest"))));
+	}
+
+	@Test
+	void executesWithDefaultLocaleConversionFormat() {
+		var results = execute(LocaleConversionTestCase.class, "testWithBcp47", Locale.class);
+
+		results.allEvents().assertStatistics(stats -> stats.started(4).succeeded(4));
+	}
+
+	@Test
+	void executesWithBcp47LocaleConversionFormat() {
+		var results = execute(Map.of(DEFAULT_LOCALE_CONVERSION_FORMAT_PROPERTY_NAME, "bcp_47"),
+			LocaleConversionTestCase.class, "testWithBcp47", Locale.class);
+
+		results.allEvents().assertStatistics(stats -> stats.started(4).succeeded(4));
+	}
+
+	@Test
+	void executesWithIso639LocaleConversionFormat() {
+		var results = execute(Map.of(DEFAULT_LOCALE_CONVERSION_FORMAT_PROPERTY_NAME, "iso_639"),
+			LocaleConversionTestCase.class, "testWithIso639", Locale.class);
+
+		results.allEvents().assertStatistics(stats -> stats.started(4).succeeded(4));
+	}
+
+	@Test
+	void reportsExceptionInStaticInitializersWithoutInvocationCountValidation() {
+		var results = executeTestsForClass(ExceptionInStaticInitializerTestCase.class);
+
+		var failure = results.containerEvents().stream() //
+				.filter(finishedWithFailure()::matches) //
+				.findAny() //
+				.orElseThrow();
+
+		var throwable = failure.getRequiredPayload(TestExecutionResult.class).getThrowable().orElseThrow();
+
+		assertThat(throwable) //
+				.isInstanceOf(ExceptionInInitializerError.class) //
+				.hasNoSuppressedExceptions();
+	}
+
+	private EngineExecutionResults execute(Map<String, String> configurationParameters, Class<?> testClass,
+			String methodName, Class<?>... methodParameterTypes) {
+		return EngineTestKit.engine(new JupiterTestEngine()) //
+				.selectors(selectMethod(testClass, methodName, ClassUtils.nullSafeToString(methodParameterTypes))) //
+				.configurationParameters(configurationParameters) //
+				.execute();
 	}
 
 	private EngineExecutionResults execute(String methodName, Class<?>... methodParameterTypes) {
 		return execute(TestCase.class, methodName, methodParameterTypes);
 	}
 
+	private EngineExecutionResults execute(Class<?> testClass, String methodName, Class<?>... methodParameterTypes) {
+		return executeTests(selectMethod(testClass, methodName, ClassUtils.nullSafeToString(methodParameterTypes)));
+	}
+
 	/**
 	 * @since 5.4
 	 */
@@ -491,7 +569,7 @@ class ParameterizedTestIntegrationTests {
 					finishedWithFailure(//
 						instanceOf(PreconditionViolationException.class), //
 						message(msg -> msg.matches(
-							"@NullSource cannot provide a null argument to method .+: the method does not declare any formal parameters.")))));
+							"@NullSource cannot provide a null argument to method .+: no formal parameters declared.")))));
 		}
 
 		@Test
@@ -631,7 +709,7 @@ class ParameterizedTestIntegrationTests {
 					finishedWithFailure(//
 						instanceOf(PreconditionViolationException.class), //
 						message(msg -> msg.matches(
-							"@EmptySource cannot provide an empty argument to method .+: the method does not declare any formal parameters.")))));
+							"@EmptySource cannot provide an empty argument to method .+: no formal parameters declared.")))));
 		}
 
 		@ParameterizedTest(name = "{1}")
@@ -874,7 +952,7 @@ class ParameterizedTestIntegrationTests {
 
 		/**
 		 * @since 5.9.1
-		 * @see https://github.com/junit-team/junit5/issues/3001
+		 * @see https://github.com/junit-team/junit-framework/issues/3001
 		 */
 		@Test
 		void duplicateMethodNames() {
@@ -883,7 +961,7 @@ class ParameterizedTestIntegrationTests {
 			// other words, we're not really testing the support for @RepeatedTest
 			// and @TestFactory, but their presence also contributes to the bug
 			// reported in #3001.
-			ParameterizedTestIntegrationTests.this.execute(selectClass(DuplicateMethodNamesMethodSourceTestCase.class))//
+			executeTestsForClass(DuplicateMethodNamesMethodSourceTestCase.class)//
 					.testEvents()//
 					.assertStatistics(stats -> stats.started(8).failed(0).finished(8));
 		}
@@ -1090,6 +1168,85 @@ class ParameterizedTestIntegrationTests {
 
 	}
 
+	@Nested
+	class UnusedArgumentsWithStrictArgumentsCountIntegrationTests {
+		@Test
+		void failsWithArgumentsSourceProvidingUnusedArguments() {
+			var results = execute(ArgumentCountValidationMode.STRICT, UnusedArgumentsTestCase.class,
+				"testWithTwoUnusedStringArgumentsProvider", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1, event(finishedWithFailure(message(String.format(
+						"Configuration error: @ParameterizedTest consumes 1 parameter but there were 2 arguments provided.%nNote: the provided arguments were [foo, unused1]")))));
+		}
+
+		@Test
+		void failsWithMethodSourceProvidingUnusedArguments() {
+			var results = execute(ArgumentCountValidationMode.STRICT, UnusedArgumentsTestCase.class,
+				"testWithMethodSourceProvidingUnusedArguments", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1, event(finishedWithFailure(message(String.format(
+						"Configuration error: @ParameterizedTest consumes 1 parameter but there were 2 arguments provided.%nNote: the provided arguments were [foo, unused1]")))));
+		}
+
+		@Test
+		void failsWithCsvSourceUnusedArgumentsAndStrictArgumentCountValidationAnnotationAttribute() {
+			var results = execute(ArgumentCountValidationMode.NONE, UnusedArgumentsTestCase.class,
+				"testWithStrictArgumentCountValidation", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1, event(finishedWithFailure(message(String.format(
+						"Configuration error: @ParameterizedTest consumes 1 parameter but there were 2 arguments provided.%nNote: the provided arguments were [foo, unused1]")))));
+		}
+
+		@Test
+		void failsWithCsvSourceUnusedArgumentsButExecutesRemainingArgumentsWhereThereIsNoUnusedArgument() {
+			var results = execute(ArgumentCountValidationMode.STRICT, UnusedArgumentsTestCase.class,
+				"testWithCsvSourceContainingDifferentNumbersOfArguments", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1, event(finishedWithFailure(message(String.format(
+						"Configuration error: @ParameterizedTest consumes 1 parameter but there were 2 arguments provided.%nNote: the provided arguments were [foo, unused1]"))))) //
+					.haveExactly(1,
+						event(test(), displayName("[2] argument=bar"), finishedWithFailure(message("bar"))));
+		}
+
+		@Test
+		void executesWithCsvSourceUnusedArgumentsAndArgumentCountValidationAnnotationAttribute() {
+			var results = execute(ArgumentCountValidationMode.NONE, UnusedArgumentsTestCase.class,
+				"testWithNoneArgumentCountValidation", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1,
+						event(test(), displayName("[1] argument=foo"), finishedWithFailure(message("foo"))));
+		}
+
+		@Test
+		void executesWithMethodSourceProvidingUnusedArguments() {
+			var results = execute(ArgumentCountValidationMode.STRICT, RepeatableSourcesTestCase.class,
+				"testWithRepeatableCsvSource", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1, event(test(), displayName("[1] argument=a"), finishedWithFailure(message("a")))) //
+					.haveExactly(1, event(test(), displayName("[2] argument=b"), finishedWithFailure(message("b"))));
+		}
+
+		@Test
+		void evaluatesArgumentsAtMostOnce() {
+			var results = execute(ArgumentCountValidationMode.STRICT, UnusedArgumentsTestCase.class,
+				"testWithEvaluationReportingArgumentsProvider", String.class);
+			results.allEvents().assertThatEvents() //
+					.haveExactly(1, event(finishedWithFailure(message(String.format(
+						"Configuration error: @ParameterizedTest consumes 1 parameter but there were 2 arguments provided.%nNote: the provided arguments were [foo, unused]")))));
+			results.allEvents().reportingEntryPublished().assertThatEvents() //
+					.haveExactly(1, event(EventConditions.reportEntry(Map.of("evaluated", "true"))));
+		}
+
+		private EngineExecutionResults execute(ArgumentCountValidationMode configurationValue, Class<?> javaClass,
+				String methodName, Class<?>... methodParameterTypes) {
+			return EngineTestKit.engine(new JupiterTestEngine()) //
+					.selectors(selectMethod(javaClass, methodName, methodParameterTypes)) //
+					.configurationParameter(ArgumentCountValidator.ARGUMENT_COUNT_VALIDATION_KEY,
+						configurationValue.name().toLowerCase()) //
+					.execute();
+		}
+	}
+
 	@Nested
 	class RepeatableSourcesIntegrationTests {
 
@@ -1199,13 +1356,31 @@ class ParameterizedTestIntegrationTests {
 		results.allEvents().assertThatEvents() //
 				.haveExactly(1, event(test(), finishedSuccessfully()));
 
-		assertTrue(AutoCloseableArgument.isClosed);
+		assertEquals(2, AutoCloseableArgument.closeCounter);
+	}
+
+	@Test
+	void doNotCloseAutoCloseableArgumentsAfterTestWhenDisabled() {
+		var results = execute("testWithAutoCloseableArgumentButDisabledCleanup", AutoCloseableArgument.class);
+		results.allEvents().assertThatEvents() //
+				.haveExactly(1, event(test(), finishedSuccessfully()));
+
+		assertEquals(0, AutoCloseableArgument.closeCounter);
+	}
+
+	@Test
+	void closeAutoCloseableArgumentsAfterTestDespiteEarlyFailure() {
+		var results = execute(FailureInBeforeEachTestCase.class, "test", AutoCloseableArgument.class);
+		results.allEvents().assertThatEvents() //
+				.haveExactly(1, event(test(), finishedWithFailure(message("beforeEach"))));
+
+		assertEquals(2, AutoCloseableArgument.closeCounter);
 	}
 
 	@Test
 	void executesTwoIterationsBasedOnIterationAndUniqueIdSelector() {
 		var methodId = uniqueIdForTestTemplateMethod(TestCase.class, "testWithThreeIterations(int)");
-		var results = execute(selectUniqueId(appendTestTemplateInvocationSegment(methodId, 3)),
+		var results = executeTests(selectUniqueId(appendTestTemplateInvocationSegment(methodId, 3)),
 			selectIteration(selectMethod(TestCase.class, "testWithThreeIterations", "int"), 1));
 
 		results.allEvents().assertThatEvents() //
@@ -1214,6 +1389,31 @@ class ParameterizedTestIntegrationTests {
 				.haveExactly(1, event(test(), displayName("[3] argument=5"), finishedWithFailure()));
 	}
 
+	@Nested
+	class SpiParameterInjectionIntegrationTests {
+
+		@Test
+		void injectsParametersIntoArgumentsProviderConstructor() {
+			execute(SpiParameterInjectionTestCase.class, "argumentsProviderWithConstructorParameter", String.class) //
+					.testEvents() //
+					.assertStatistics(it -> it.succeeded(1));
+		}
+
+		@Test
+		void injectsParametersIntoArgumentConverterConstructor() {
+			execute(SpiParameterInjectionTestCase.class, "argumentConverterWithConstructorParameter", String.class) //
+					.testEvents() //
+					.assertStatistics(it -> it.succeeded(1));
+		}
+
+		@Test
+		void injectsParametersIntoArgumentsAggregatorConstructor() {
+			execute(SpiParameterInjectionTestCase.class, "argumentsAggregatorWithConstructorParameter", String.class) //
+					.testEvents() //
+					.assertStatistics(it -> it.succeeded(1));
+		}
+	}
+
 	// -------------------------------------------------------------------------
 
 	static class TestCase {
@@ -1305,7 +1505,13 @@ class ParameterizedTestIntegrationTests {
 		@ParameterizedTest
 		@ArgumentsSource(AutoCloseableArgumentProvider.class)
 		void testWithAutoCloseableArgument(AutoCloseableArgument autoCloseable) {
-			assertFalse(AutoCloseableArgument.isClosed);
+			assertEquals(0, AutoCloseableArgument.closeCounter);
+		}
+
+		@ParameterizedTest(autoCloseArguments = false)
+		@ArgumentsSource(AutoCloseableArgumentProvider.class)
+		void testWithAutoCloseableArgumentButDisabledCleanup(AutoCloseableArgument autoCloseable) {
+			assertEquals(0, AutoCloseableArgument.closeCounter);
 		}
 
 		@ParameterizedTest
@@ -1315,6 +1521,7 @@ class ParameterizedTestIntegrationTests {
 		}
 	}
 
+	@SuppressWarnings("JUnitMalformedDeclaration")
 	static class NullSourceTestCase {
 
 		@ParameterizedTest
@@ -1350,6 +1557,7 @@ class ParameterizedTestIntegrationTests {
 
 	}
 
+	@SuppressWarnings("JUnitMalformedDeclaration")
 	static class EmptySourceTestCase {
 
 		@ParameterizedTest
@@ -1505,6 +1713,7 @@ class ParameterizedTestIntegrationTests {
 
 	}
 
+	@SuppressWarnings("JUnitMalformedDeclaration")
 	static class NullAndEmptySourceTestCase {
 
 		@ParameterizedTest
@@ -1546,6 +1755,7 @@ class ParameterizedTestIntegrationTests {
 
 	}
 
+	@SuppressWarnings("JUnitMalformedDeclaration")
 	@TestMethodOrder(OrderAnnotation.class)
 	static class MethodSourceTestCase {
 
@@ -1726,7 +1936,7 @@ class ParameterizedTestIntegrationTests {
 
 	/**
 	 * @since 5.9.1
-	 * @see https://github.com/junit-team/junit5/issues/3001
+	 * @see https://github.com/junit-team/junit-framework/issues/3001
 	 */
 	static class DuplicateMethodNamesMethodSourceTestCase {
 
@@ -1977,6 +2187,41 @@ class ParameterizedTestIntegrationTests {
 		static Supplier<Stream<Arguments>> unusedArgumentsProviderField = //
 			() -> Stream.of(arguments("foo", "unused1"), arguments("bar", "unused2"));
 
+		@ParameterizedTest(argumentCountValidation = ArgumentCountValidationMode.STRICT)
+		@CsvSource({ "foo, unused1" })
+		void testWithStrictArgumentCountValidation(String argument) {
+			fail(argument);
+		}
+
+		@ParameterizedTest(argumentCountValidation = ArgumentCountValidationMode.NONE)
+		@CsvSource({ "foo, unused1" })
+		void testWithNoneArgumentCountValidation(String argument) {
+			fail(argument);
+		}
+
+		@ParameterizedTest
+		@CsvSource({ "foo, unused1", "bar" })
+		void testWithCsvSourceContainingDifferentNumbersOfArguments(String argument) {
+			fail(argument);
+		}
+
+		@ParameterizedTest
+		@ArgumentsSource(EvaluationReportingArgumentsProvider.class)
+		void testWithEvaluationReportingArgumentsProvider(String argument) {
+			fail(argument);
+		}
+
+		private static class EvaluationReportingArgumentsProvider implements ArgumentsProvider {
+
+			@Override
+			public Stream<? extends Arguments> provideArguments(ParameterDeclarations parameters,
+					ExtensionContext context) {
+				return Stream.of(() -> {
+					context.publishReportEntry("evaluated", "true");
+					return List.of("foo", "unused").toArray();
+				});
+			}
+		}
 	}
 
 	static class LifecycleTestCase {
@@ -2117,6 +2362,7 @@ class ParameterizedTestIntegrationTests {
 		@interface TwoMethodSources {
 		}
 
+		@SuppressWarnings("JUnitMalformedDeclaration")
 		@ParameterizedTest
 		@TwoMethodSources
 		void testWithRepeatableMethodSourceAsMetaAnnotation(String argument) {
@@ -2211,10 +2457,126 @@ class ParameterizedTestIntegrationTests {
 		}
 	}
 
+	static class SpiParameterInjectionTestCase {
+
+		@RegisterExtension
+		static final ParameterResolver spiParameterResolver = new ParameterResolver() {
+
+			@Override
+			public boolean supportsParameter(ParameterContext parameterContext, ExtensionContext extensionContext)
+					throws ParameterResolutionException {
+				return parameterContext.getDeclaringExecutable() instanceof Constructor //
+						&& String.class.equals(parameterContext.getParameter().getType());
+			}
+
+			@Override
+			public Object resolveParameter(ParameterContext parameterContext, ExtensionContext extensionContext)
+					throws ParameterResolutionException {
+				return "resolved value";
+			}
+		};
+
+		@ParameterizedTest
+		@ArgumentsSource(ArgumentsProviderWithConstructorParameter.class)
+		void argumentsProviderWithConstructorParameter(String argument) {
+			assertEquals("resolved value", argument);
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = "value")
+		void argumentConverterWithConstructorParameter(
+				@ConvertWith(ArgumentConverterWithConstructorParameter.class) String argument) {
+			assertEquals("resolved value", argument);
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = "value")
+		void argumentsAggregatorWithConstructorParameter(
+				@AggregateWith(ArgumentsAggregatorWithConstructorParameter.class) String argument) {
+			assertEquals("resolved value", argument);
+		}
+
+		record ArgumentsProviderWithConstructorParameter(String value) implements ArgumentsProvider {
+
+			@Override
+			public Stream<? extends Arguments> provideArguments(ParameterDeclarations parameters,
+					ExtensionContext context) {
+				return Stream.of(arguments(value));
+			}
+		}
+
+		record ArgumentConverterWithConstructorParameter(String value) implements ArgumentConverter {
+
+			@Override
+			public Object convert(Object source, ParameterContext context) throws ArgumentConversionException {
+				return value;
+			}
+		}
+
+		static class ArgumentsAggregatorWithConstructorParameter extends SimpleArgumentsAggregator {
+
+			private final String value;
+
+			public ArgumentsAggregatorWithConstructorParameter(String value) {
+				this.value = value;
+			}
+
+			@Override
+			protected Object aggregateArguments(ArgumentsAccessor accessor, Class<?> targetType,
+					AnnotatedElementContext context, int parameterIndex) throws ArgumentsAggregationException {
+				return this.value;
+			}
+		}
+	}
+
+	static class ZeroInvocationsTestCase {
+
+		@ParameterizedTest
+		@MethodSource("zeroArgumentsProvider")
+		void testThatRequiresInvocations(String argument) {
+			fail("This test should not be executed, because no arguments are provided.");
+		}
+
+		@ParameterizedTest(allowZeroInvocations = true)
+		@MethodSource("zeroArgumentsProvider")
+		void testThatDoesNotRequireInvocations(String argument) {
+			fail("This test should not be executed, because no arguments are provided.");
+		}
+
+		@ParameterizedTest(allowZeroInvocations = true)
+		@SuppressWarnings("JUnitMalformedDeclaration")
+		void testThatHasNoArgumentsSource(String argument) {
+			fail("This test should not be executed, because no arguments source is declared.");
+		}
+
+		public static Stream<Arguments> zeroArgumentsProvider() {
+			return Stream.empty();
+		}
+	}
+
+	static class LocaleConversionTestCase {
+
+		@ParameterizedTest
+		@ValueSource(strings = "en-US")
+		void testWithBcp47(Locale locale) {
+			assertEquals("en", locale.getLanguage());
+			assertEquals("US", locale.getCountry());
+		}
+
+		@ParameterizedTest
+		@ValueSource(strings = "en-US")
+		void testWithIso639(Locale locale) {
+			assertEquals("en-us", locale.getLanguage());
+			assertEquals("", locale.getCountry());
+		}
+
+	}
+
 	private static class TwoSingleStringArgumentsProvider implements ArgumentsProvider {
 
 		@Override
-		public Stream<? extends Arguments> provideArguments(ExtensionContext context) {
+		public Stream<? extends Arguments> provideArguments(ParameterDeclarations parameters,
+				ExtensionContext context) {
 			return Stream.of(arguments("foo"), arguments("bar"));
 		}
 	}
@@ -2222,7 +2584,8 @@ class ParameterizedTestIntegrationTests {
 	private static class TwoUnusedStringArgumentsProvider implements ArgumentsProvider {
 
 		@Override
-		public Stream<? extends Arguments> provideArguments(ExtensionContext context) {
+		public Stream<? extends Arguments> provideArguments(ParameterDeclarations parameters,
+				ExtensionContext context) {
 			return Stream.of(arguments("foo", "unused1"), arguments("bar", "unused2"));
 		}
 	}
@@ -2235,11 +2598,11 @@ class ParameterizedTestIntegrationTests {
 		}
 	}
 
-	private static class StringAggregator implements ArgumentsAggregator {
+	private static class StringAggregator extends SimpleArgumentsAggregator {
 
 		@Override
-		public Object aggregateArguments(ArgumentsAccessor accessor, ParameterContext context)
-				throws ArgumentsAggregationException {
+		protected Object aggregateArguments(ArgumentsAccessor accessor, Class<?> targetType,
+				AnnotatedElementContext context, int parameterIndex) throws ArgumentsAggregationException {
 			return accessor.getString(0) + accessor.getString(1);
 		}
 	}
@@ -2255,18 +2618,19 @@ class ParameterizedTestIntegrationTests {
 	private static class AutoCloseableArgumentProvider implements ArgumentsProvider {
 
 		@Override
-		public Stream<? extends Arguments> provideArguments(ExtensionContext context) {
-			return Stream.of(arguments(new AutoCloseableArgument()));
+		public Stream<? extends Arguments> provideArguments(ParameterDeclarations parameters,
+				ExtensionContext context) {
+			return Stream.of(arguments(new AutoCloseableArgument(), Named.of("unused", new AutoCloseableArgument())));
 		}
 	}
 
 	static class AutoCloseableArgument implements AutoCloseable {
 
-		static boolean isClosed = false;
+		static int closeCounter = 0;
 
 		@Override
 		public void close() {
-			isClosed = true;
+			closeCounter++;
 		}
 	}
 
@@ -2283,4 +2647,39 @@ class ParameterizedTestIntegrationTests {
 		}
 	}
 
+	static class FailureInBeforeEachTestCase {
+
+		@BeforeEach
+		void beforeEach() {
+			fail("beforeEach");
+		}
+
+		@ParameterizedTest
+		@ArgumentsSource(AutoCloseableArgumentProvider.class)
+		void test(AutoCloseableArgument autoCloseable) {
+			assertNotNull(autoCloseable);
+			assertEquals(0, AutoCloseableArgument.closeCounter);
+		}
+	}
+
+	static class ExceptionInStaticInitializerTestCase {
+
+		static {
+			//noinspection ConstantValue
+			if (true)
+				throw new RuntimeException("boom");
+		}
+
+		private static Stream<String> getArguments() {
+			return Stream.of("foo", "bar");
+		}
+
+		@ParameterizedTest
+		@MethodSource("getArguments")
+		void test(String value) {
+			fail("should not be called: " + value);
+		}
+
+	}
+
 }
