@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -10,7 +10,6 @@
 
 package org.junit.platform.commons.util;
 
-import static java.util.stream.Collectors.toList;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
@@ -94,7 +93,7 @@ class AnnotationUtilsTests {
 	}
 
 	/**
-	 * Test for https://github.com/junit-team/junit5/issues/1133
+	 * Test for https://github.com/junit-team/junit-framework/issues/1133
 	 */
 	@Test
 	void findInheritedAnnotationMetaPresentOnNonInheritedComposedAnnotationPresentOnSuperclass() {
@@ -273,8 +272,7 @@ class AnnotationUtilsTests {
 	}
 
 	private void assertTagsFound(Class<?> clazz, String... tags) {
-		assertEquals(List.of(tags),
-			findRepeatableAnnotations(clazz, Tag.class).stream().map(Tag::value).collect(toList()),
+		assertEquals(List.of(tags), findRepeatableAnnotations(clazz, Tag.class).stream().map(Tag::value).toList(),
 			() -> "Tags found for class " + clazz.getName());
 	}
 
@@ -332,7 +330,7 @@ class AnnotationUtilsTests {
 
 	private void assertExtensionsFound(Class<?> clazz, String... tags) {
 		assertEquals(List.of(tags),
-			findRepeatableAnnotations(clazz, ExtendWith.class).stream().map(ExtendWith::value).collect(toList()),
+			findRepeatableAnnotations(clazz, ExtendWith.class).stream().map(ExtendWith::value).toList(),
 			() -> "Extensions found for class " + clazz.getName());
 	}
 
@@ -385,12 +383,12 @@ class AnnotationUtilsTests {
 		var methods = findAnnotatedMethods(ClassWithAnnotatedMethods.class, Annotation1.class, TOP_DOWN);
 
 		assertEquals(3, methods.size());
-		assertEquals(superMethod, methods.get(0));
+		assertEquals(superMethod, methods.getFirst());
 		assertThat(methods.subList(1, 3)).containsOnly(method1, method3);
 	}
 
-	/**
-	 * @see https://github.com/junit-team/junit5/issues/3553
+	/*
+	 * see https://github.com/junit-team/junit-framework/issues/3553
 	 */
 	@Test
 	void findAnnotatedMethodsDoesNotAllowInstanceMethodToHideStaticMethod() throws Exception {
@@ -477,7 +475,7 @@ class AnnotationUtilsTests {
 		var fields = findAnnotatedFields(ClassWithAnnotatedFields.class, Annotation1.class, isStringField, TOP_DOWN);
 
 		assertEquals(3, fields.size());
-		assertEquals(superField, fields.get(0));
+		assertEquals(superField, fields.getFirst());
 		assertThat(fields.subList(1, 3)).containsOnly(field1, field3);
 	}
 
@@ -523,8 +521,8 @@ class AnnotationUtilsTests {
 		return values.stream().map(String::valueOf).toList();
 	}
 
-	/**
-	 * @see https://github.com/junit-team/junit5/issues/3553
+	/*
+	 * see https://github.com/junit-team/junit-framework/issues/3553
 	 */
 	@Test
 	void findAnnotatedFieldsDoesNotAllowInstanceFieldToHideStaticField() throws Exception {
@@ -610,7 +608,7 @@ class AnnotationUtilsTests {
 	}
 
 	private List<String> asNames(List<Field> fields) {
-		return fields.stream().map(Field::getName).collect(toList());
+		return fields.stream().map(Field::getName).toList();
 	}
 
 	// -------------------------------------------------------------------------
