@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -10,8 +10,9 @@
 
 package org.junit.platform.engine.discovery;
 
+import static org.junit.jupiter.api.EqualsAndHashCodeAssertions.assertEqualsAndHashCode;
+
 import org.junit.jupiter.api.Test;
-import org.junit.platform.AbstractEqualsAndHashCodeTests;
 import org.junit.platform.engine.UniqueId;
 
 /**
@@ -20,7 +21,7 @@ import org.junit.platform.engine.UniqueId;
  * @since 1.3
  * @see DiscoverySelectorsTests
  */
-class UniqueIdSelectorTests extends AbstractEqualsAndHashCodeTests {
+class UniqueIdSelectorTests {
 
 	@Test
 	void equalsAndHashCode() {
