@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -10,9 +10,11 @@
 
 package org.junit.platform.suite.engine;
 
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.platform.engine.discovery.DiscoverySelectors.selectClass;
 import static org.junit.platform.engine.discovery.DiscoverySelectors.selectUniqueId;
 import static org.junit.platform.launcher.TagFilter.excludeTags;
+import static org.junit.platform.launcher.core.OutputDirectoryCreators.hierarchicalOutputDirectoryCreator;
 import static org.junit.platform.suite.engine.SuiteEngineDescriptor.ENGINE_ID;
 import static org.junit.platform.testkit.engine.EventConditions.container;
 import static org.junit.platform.testkit.engine.EventConditions.displayName;
@@ -23,24 +25,44 @@ import static org.junit.platform.testkit.engine.EventConditions.finishedWithFail
 import static org.junit.platform.testkit.engine.EventConditions.test;
 import static org.junit.platform.testkit.engine.TestExecutionResultConditions.instanceOf;
 import static org.junit.platform.testkit.engine.TestExecutionResultConditions.message;
+import static org.mockito.ArgumentMatchers.same;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.verify;
+
+import java.nio.file.Path;
 
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 import org.junit.jupiter.engine.descriptor.ClassTestDescriptor;
 import org.junit.jupiter.engine.descriptor.JupiterEngineDescriptor;
 import org.junit.jupiter.engine.descriptor.TestMethodTestDescriptor;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.ValueSource;
+import org.junit.platform.engine.ConfigurationParameters;
+import org.junit.platform.engine.DiscoveryIssue;
+import org.junit.platform.engine.DiscoveryIssue.Severity;
+import org.junit.platform.engine.EngineExecutionListener;
+import org.junit.platform.engine.ExecutionRequest;
 import org.junit.platform.engine.FilterResult;
+import org.junit.platform.engine.OutputDirectoryCreator;
 import org.junit.platform.engine.UniqueId;
+import org.junit.platform.engine.support.descriptor.ClassSource;
 import org.junit.platform.engine.support.descriptor.MethodSource;
+import org.junit.platform.engine.support.store.Namespace;
+import org.junit.platform.engine.support.store.NamespacedHierarchicalStore;
 import org.junit.platform.launcher.PostDiscoveryFilter;
+import org.junit.platform.launcher.core.NamespacedHierarchicalStoreProviders;
 import org.junit.platform.suite.api.SelectClasses;
 import org.junit.platform.suite.api.Suite;
 import org.junit.platform.suite.engine.testcases.ConfigurationSensitiveTestCase;
 import org.junit.platform.suite.engine.testcases.DynamicTestsTestCase;
+import org.junit.platform.suite.engine.testcases.ErroneousTestCase;
 import org.junit.platform.suite.engine.testcases.JUnit4TestsTestCase;
 import org.junit.platform.suite.engine.testcases.MultipleTestsTestCase;
 import org.junit.platform.suite.engine.testcases.SingleTestTestCase;
 import org.junit.platform.suite.engine.testcases.TaggedTestTestCase;
 import org.junit.platform.suite.engine.testsuites.AbstractSuite;
+import org.junit.platform.suite.engine.testsuites.BlankSuiteDisplayNameSuite;
 import org.junit.platform.suite.engine.testsuites.ConfigurationSuite;
 import org.junit.platform.suite.engine.testsuites.CyclicSuite;
 import org.junit.platform.suite.engine.testsuites.DynamicSuite;
@@ -49,6 +71,8 @@ import org.junit.platform.suite.engine.testsuites.EmptyDynamicTestSuite;
 import org.junit.platform.suite.engine.testsuites.EmptyDynamicTestWithFailIfNoTestFalseSuite;
 import org.junit.platform.suite.engine.testsuites.EmptyTestCaseSuite;
 import org.junit.platform.suite.engine.testsuites.EmptyTestCaseWithFailIfNoTestFalseSuite;
+import org.junit.platform.suite.engine.testsuites.ErroneousTestSuite;
+import org.junit.platform.suite.engine.testsuites.InheritedSuite;
 import org.junit.platform.suite.engine.testsuites.MultiEngineSuite;
 import org.junit.platform.suite.engine.testsuites.MultipleSuite;
 import org.junit.platform.suite.engine.testsuites.NestedSuite;
@@ -57,7 +81,9 @@ import org.junit.platform.suite.engine.testsuites.SelectClassesSuite;
 import org.junit.platform.suite.engine.testsuites.SelectMethodsSuite;
 import org.junit.platform.suite.engine.testsuites.SuiteDisplayNameSuite;
 import org.junit.platform.suite.engine.testsuites.SuiteSuite;
+import org.junit.platform.suite.engine.testsuites.SuiteWithErroneousTestSuite;
 import org.junit.platform.suite.engine.testsuites.ThreePartCyclicSuite;
+import org.junit.platform.suite.engine.testsuites.WhitespaceSuiteDisplayNameSuite;
 import org.junit.platform.testkit.engine.EngineTestKit;
 
 /**
@@ -65,15 +91,25 @@ import org.junit.platform.testkit.engine.EngineTestKit;
  */
 class SuiteEngineTests {
 
-	@Test
-	void selectClasses() {
+	@TempDir
+	private Path outputDir;
+
+	@ParameterizedTest
+	@ValueSource(classes = { SelectClassesSuite.class, InheritedSuite.class })
+	void selectClasses(Class<?> suiteClass) {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(SelectClassesSuite.class))
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(suiteClass));
+			var testKit = builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.isEmpty();
+
+		testKit
 				.execute()
 				.testEvents()
 				.assertThatEvents()
-				.haveExactly(1, event(test(SelectClassesSuite.class.getName()), finishedSuccessfully()))
+				.haveExactly(1, event(test(suiteClass.getName()), finishedSuccessfully()))
 				.haveExactly(1, event(test(SingleTestTestCase.class.getName()), finishedSuccessfully()));
 		// @formatter:on
 	}
@@ -81,8 +117,13 @@ class SuiteEngineTests {
 	@Test
 	void selectMethods() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(SelectMethodsSuite.class))
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(SelectMethodsSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.isEmpty();
+
+		testKit
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -106,8 +147,13 @@ class SuiteEngineTests {
 	@Test
 	void abstractSuiteIsNotExecuted() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(AbstractSuite.class))
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(AbstractSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.isEmpty();
+
+		testKit
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -118,8 +164,18 @@ class SuiteEngineTests {
 	@Test
 	void privateSuiteIsNotExecuted() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(PrivateSuite.class))
+		var message = "@Suite class '%s' must not be private. It will not be executed."
+				.formatted(PrivateSuite.class.getName());
+		var issue = DiscoveryIssue.builder(Severity.WARNING, message)
+				.source(ClassSource.from(PrivateSuite.class))
+				.build();
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(PrivateSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.containsExactly(issue);
+
+		testKit
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -128,10 +184,86 @@ class SuiteEngineTests {
 	}
 
 	@Test
-	void innerSuiteIsNotExecuted() {
+	void abstractPrivateSuiteIsNotExecuted() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(InnerSuite.class))
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(AbstractPrivateSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.isEmpty();
+
+		testKit
+				.execute()
+				.testEvents()
+				.assertThatEvents()
+				.isEmpty();
+		// @formatter:on
+	}
+
+	@ParameterizedTest
+	@ValueSource(classes = { InnerSuite.class, AbstractInnerSuite.class })
+	void innerSuiteIsNotExecuted(Class<?> suiteClass) {
+		// @formatter:off
+		var message = "@Suite class '%s' must not be an inner class. Did you forget to declare it static? It will not be executed."
+				.formatted(suiteClass.getName());
+		var issue = DiscoveryIssue.builder(Severity.WARNING, message)
+				.source(ClassSource.from(suiteClass))
+				.build();
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(suiteClass));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.containsExactly(issue);
+
+		testKit
+				.execute()
+				.testEvents()
+				.assertThatEvents()
+				.isEmpty();
+		// @formatter:on
+	}
+
+	@Test
+	void localSuiteIsNotExecuted() {
+
+		@Suite
+		@SelectClasses(names = "org.junit.platform.suite.engine.testcases.SingleTestTestCase")
+		class LocalSuite {
+		}
+
+		// @formatter:off
+		var message = "@Suite class '%s' must not be a local class. It will not be executed."
+				.formatted(LocalSuite.class.getName());
+		var issue = DiscoveryIssue.builder(Severity.WARNING, message)
+				.source(ClassSource.from(LocalSuite.class))
+				.build();
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(LocalSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.containsExactly(issue);
+
+		testKit
+				.execute()
+				.testEvents()
+				.assertThatEvents()
+				.isEmpty();
+		// @formatter:on
+	}
+
+	@Test
+	void anonymousSuiteIsNotExecuted() {
+		var object = new Object() {
+		};
+
+		// @formatter:off
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(object.getClass()));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.isEmpty();
+
+		testKit
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -166,8 +298,9 @@ class SuiteEngineTests {
 	@Test
 	void suiteSuite() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(SuiteSuite.class))
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(SuiteSuite.class));
+			builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir))
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -182,8 +315,9 @@ class SuiteEngineTests {
 		// @formatter:off
 		UniqueId uniqId = UniqueId.forEngine(ENGINE_ID)
 				.append(SuiteTestDescriptor.SEGMENT_TYPE, SelectClassesSuite.class.getName());
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectUniqueId(uniqId))
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectUniqueId(uniqId));
+			builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir))
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -236,9 +370,10 @@ class SuiteEngineTests {
 				.append(ClassTestDescriptor.SEGMENT_TYPE, MultipleTestsTestCase.class.getName())
 				.append(TestMethodTestDescriptor.SEGMENT_TYPE, "test()");
 
-		EngineTestKit.engine(ENGINE_ID)
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
 				.selectors(selectUniqueId(uniqueId))
-				.selectors(selectClass(SelectClassesSuite.class))
+				.selectors(selectClass(SelectClassesSuite.class));
+			builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir))
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -389,8 +524,24 @@ class SuiteEngineTests {
 	@Test
 	void cyclicSuite() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(CyclicSuite.class))
+		var expectedUniqueId = UniqueId.forEngine(ENGINE_ID)
+				.append(SuiteTestDescriptor.SEGMENT_TYPE, CyclicSuite.class.getName())
+				.appendEngine(ENGINE_ID)
+				.append(SuiteTestDescriptor.SEGMENT_TYPE, CyclicSuite.class.getName());
+		var message = "The suite configuration of [%s] resulted in a cycle [%s] and will not be discovered a second time."
+				.formatted(CyclicSuite.class.getName(), expectedUniqueId);
+		var issue = DiscoveryIssue.builder(Severity.INFO, message)
+				.source(ClassSource.from(CyclicSuite.class))
+				.build();
+
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(CyclicSuite.class));
+			var testKit = builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir));
+
+		assertThat(testKit.discover().getDiscoveryIssues())
+				.containsExactly(issue);
+
+		testKit
 				.execute()
 				.allEvents()
 				.assertThatEvents()
@@ -415,8 +566,9 @@ class SuiteEngineTests {
 	@Test
 	void threePartCyclicSuite() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(ThreePartCyclicSuite.PartA.class))
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(ThreePartCyclicSuite.PartA.class));
+			builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir))
 				.execute()
 				.allEvents()
 				.assertThatEvents()
@@ -427,8 +579,9 @@ class SuiteEngineTests {
 	@Test
 	void selectByIdentifier() {
 		// @formatter:off
-		EngineTestKit.engine(ENGINE_ID)
-				.selectors(selectClass(SelectByIdentifierSuite.class))
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(SelectByIdentifierSuite.class));
+			builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir))
 				.execute()
 				.testEvents()
 				.assertThatEvents()
@@ -437,6 +590,103 @@ class SuiteEngineTests {
 		// @formatter:on
 	}
 
+	@Test
+	void passesOutputDirectoryCreatorToEnginesInSuite() {
+		// @formatter:off
+		EngineTestKit.Builder builder = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(SelectClassesSuite.class));
+			builder.outputDirectoryCreator(hierarchicalOutputDirectoryCreator(outputDir))
+				.execute()
+				.testEvents()
+				.assertThatEvents()
+				.haveExactly(1, event(test(SingleTestTestCase.class.getName()), finishedSuccessfully()));
+		// @formatter:on
+
+		assertThat(outputDir).isDirectoryRecursivelyContaining("glob:**/test.txt");
+	}
+
+	@Test
+	void discoveryIssueOfNestedTestEnginesAreReported() throws Exception {
+		// @formatter:off
+		var testKit = EngineTestKit.engine(ENGINE_ID)
+				.selectors(selectClass(SuiteWithErroneousTestSuite.class));
+
+		var discoveryIssues = testKit.discover().getDiscoveryIssues();
+		assertThat(discoveryIssues).hasSize(1);
+
+		var issue = discoveryIssues.getFirst();
+		assertThat(issue.message()) //
+				.startsWith("[junit-jupiter] @BeforeAll method") //
+				.endsWith(" (via @Suite %s > %s).".formatted(SuiteWithErroneousTestSuite.class.getName(),
+						ErroneousTestSuite.class.getName()));
+
+		var method = ErroneousTestCase.class.getDeclaredMethod("nonStaticLifecycleMethod");
+		assertThat(issue.source()).contains(MethodSource.from(method));
+
+		testKit
+				.execute()
+				.testEvents()
+				.assertThatEvents()
+				.isEmpty();
+		// @formatter:on
+	}
+
+	@Suite
+	@SelectClasses(SingleTestTestCase.class)
+	abstract private static class AbstractPrivateSuite {
+	}
+
+	@Test
+	void suiteEnginePassesRequestLevelStoreToSuiteTestDescriptors() {
+		UniqueId engineId = UniqueId.forEngine(SuiteEngineDescriptor.ENGINE_ID);
+		SuiteEngineDescriptor engineDescriptor = new SuiteEngineDescriptor(engineId);
+
+		SuiteTestDescriptor mockDescriptor = mock(SuiteTestDescriptor.class);
+		engineDescriptor.addChild(mockDescriptor);
+
+		EngineExecutionListener listener = mock(EngineExecutionListener.class);
+		NamespacedHierarchicalStore<Namespace> requestLevelStore = NamespacedHierarchicalStoreProviders.dummyNamespacedHierarchicalStore();
+
+		ExecutionRequest request = ExecutionRequest.create(engineDescriptor, listener,
+			mock(ConfigurationParameters.class), mock(OutputDirectoryCreator.class), requestLevelStore);
+
+		new SuiteTestEngine().execute(request);
+
+		verify(mockDescriptor).execute(same(listener), same(requestLevelStore));
+	}
+
+	@Test
+	void blankSuiteDisplayNameGeneratesWarning() {
+		var expectedMessage = "@SuiteDisplayName on %s must be declared with a non-blank value.".formatted(
+			BlankSuiteDisplayNameSuite.class.getName());
+		var expectedIssue = DiscoveryIssue.builder(Severity.WARNING, expectedMessage).source(
+			ClassSource.from(BlankSuiteDisplayNameSuite.class)).build();
+
+		var testKit = EngineTestKit.engine(ENGINE_ID).selectors(selectClass(BlankSuiteDisplayNameSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues()).contains(expectedIssue);
+	}
+
+	@Test
+	void whitespaceSuiteDisplayNameGeneratesWarning() {
+		var expectedMessage = "@SuiteDisplayName on %s must be declared with a non-blank value.".formatted(
+			WhitespaceSuiteDisplayNameSuite.class.getName());
+		var expectedIssue = DiscoveryIssue.builder(Severity.WARNING, expectedMessage).source(
+			ClassSource.from(WhitespaceSuiteDisplayNameSuite.class)).build();
+
+		var testKit = EngineTestKit.engine(ENGINE_ID).selectors(selectClass(WhitespaceSuiteDisplayNameSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues()).contains(expectedIssue);
+	}
+
+	@Test
+	void validSuiteDisplayNameGeneratesNoWarning() {
+		var testKit = EngineTestKit.engine(ENGINE_ID).selectors(selectClass(SuiteDisplayNameSuite.class));
+
+		assertThat(testKit.discover().getDiscoveryIssues()) //
+				.noneMatch(issue -> issue.message().contains("@SuiteDisplayName"));
+	}
+
 	@Suite
 	@SelectClasses(SingleTestTestCase.class)
 	private static class PrivateSuite {
@@ -444,7 +694,12 @@ class SuiteEngineTests {
 
 	@Suite
 	@SelectClasses(names = "org.junit.platform.suite.engine.testcases.SingleTestTestCase")
-	private class InnerSuite {
+	abstract class AbstractInnerSuite {
+	}
+
+	@Suite
+	@SelectClasses(names = "org.junit.platform.suite.engine.testcases.SingleTestTestCase")
+	class InnerSuite {
 	}
 
 }
