@@ -1,5 +1,5 @@
 /*
- * Copyright 2015-2024 the original author or authors.
+ * Copyright 2015-2026 the original author or authors.
  *
  * All rights reserved. This program and the accompanying materials are
  * made available under the terms of the Eclipse Public License v2.0 which
@@ -15,6 +15,10 @@ import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertLinesMatch;
 import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.parallel.ExecutionMode.CONCURRENT;
+import static org.junit.jupiter.api.parallel.ExecutionMode.SAME_THREAD;
+import static platform.tooling.support.tests.Projects.copyToWorkspace;
+import static platform.tooling.support.tests.Projects.getSourceDirectory;
 
 import java.io.File;
 import java.io.IOException;
@@ -25,31 +29,37 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.stream.Stream;
 
-import de.sormuras.bartholdy.Result;
-import de.sormuras.bartholdy.jdk.Jar;
-import de.sormuras.bartholdy.jdk.Javac;
-import de.sormuras.bartholdy.tool.Java;
-
-import org.junit.jupiter.api.Disabled;
+import org.junit.jupiter.api.BeforeAll;
 import org.junit.jupiter.api.MethodOrderer;
 import org.junit.jupiter.api.Order;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.TestMethodOrder;
-import org.junit.jupiter.api.parallel.ResourceLock;
+import org.junit.jupiter.api.io.TempDir;
+import org.junit.jupiter.api.parallel.Execution;
+import org.junit.platform.tests.process.OutputFiles;
+import org.junit.platform.tests.process.ProcessResult;
 import org.opentest4j.TestAbortedException;
 
 import platform.tooling.support.Helper;
 import platform.tooling.support.MavenRepo;
-import platform.tooling.support.Request;
+import platform.tooling.support.ProcessStarters;
 import platform.tooling.support.ThirdPartyJars;
 
 /**
  * @since 1.4
  */
 @TestMethodOrder(MethodOrderer.OrderAnnotation.class)
+@Execution(CONCURRENT)
 class StandaloneTests {
 
-	@ResourceLock(Projects.STANDALONE)
+	@TempDir
+	static Path workspace;
+
+	@BeforeAll
+	static void prepareWorkspace() throws IOException {
+		copyToWorkspace(Projects.STANDALONE, workspace);
+	}
+
 	@Test
 	void jarFileWithoutCompiledModuleDescriptorClass() throws Exception {
 		var jar = MavenRepo.jar("junit-platform-console-standalone");
@@ -66,17 +76,16 @@ class StandaloneTests {
 		assertTrue(found.isEmpty(), jar + " must not contain any " + name + " files: " + found);
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
-	void listAllObservableEngines() {
-		var result = Request.builder() //
-				.setTool(new Java()) //
-				.setProject(Projects.STANDALONE) //
-				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone")) //
-				.addArguments("engines", "--disable-ansi-colors", "--disable-banner").build() //
-				.run(false);
+	void listAllObservableEngines(@FilePrefix("java") OutputFiles outputFiles) throws Exception {
+		var result = ProcessStarters.java() //
+				.workingDir(getSourceDirectory(Projects.STANDALONE)) //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
+				.addArguments("engines", "--disable-ansi-colors", "--disable-banner") //
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(0, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(0, result.exitCode());
 
 		var jupiterVersion = Helper.version("junit-jupiter-engine");
 		var suiteVersion = Helper.version("junit-platform-suite-engine");
@@ -86,22 +95,20 @@ class StandaloneTests {
 				junit-platform-suite (org.junit.platform:junit-platform-suite-engine:%s)
 				junit-vintage (org.junit.vintage:junit-vintage-engine:%s)
 				""".formatted(jupiterVersion, suiteVersion, vintageVersion).lines(), //
-			result.getOutput("out").lines());
+			result.stdOut().lines());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
-	void printVersionViaJar() {
-		var result = Request.builder() //
-				.setTool(new Java()) //
-				.setProject(Projects.STANDALONE) //
-				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone")) //
+	void printVersionViaJar(@FilePrefix("java") OutputFiles outputFiles) throws Exception {
+		var result = ProcessStarters.java() //
+				.workingDir(getSourceDirectory(Projects.STANDALONE)) //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
 				.addArguments("--version", "--disable-ansi-colors") //
 				.putEnvironment("CLICOLOR_FORCE", "1") // enable ANSI colors by default (see https://picocli.info/#_heuristics_for_enabling_ansi)
-				.build() //
-				.run();
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(0, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(0, result.exitCode());
 
 		var version = Helper.version("junit-platform-console");
 		assertLinesMatch("""
@@ -109,30 +116,31 @@ class StandaloneTests {
 				JVM: .*
 				OS: .*
 				""".formatted(version).lines(), //
-			result.getOutputLines("out").stream());
+			result.stdOut().lines());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
-	void printVersionViaModule() {
+	void printVersionViaModule(@FilePrefix("java") OutputFiles outputFiles) throws Exception {
 		var junitJars = Stream.of("junit-platform-console", "junit-platform-reporting", "junit-platform-engine",
 			"junit-platform-launcher", "junit-platform-commons") //
 				.map(MavenRepo::jar);
-		var thirdPartyJars = Stream.of(ThirdPartyJars.find("org.opentest4j", "opentest4j"));
+		var thirdPartyJars = Stream.of( //
+			ThirdPartyJars.find("org.opentest4j", "opentest4j"), //
+			ThirdPartyJars.find("org.opentest4j.reporting", "open-test-reporting-tooling-spi") //
+		);
 		var modulePath = Stream.concat(junitJars, thirdPartyJars) //
 				.map(String::valueOf) //
 				.collect(joining(File.pathSeparator));
-		var result = Request.builder() //
-				.setTool(new Java()) //
-				.setProject(Projects.STANDALONE) //
+		var result = ProcessStarters.java() //
+				.workingDir(getSourceDirectory(Projects.STANDALONE)) //
 				.addArguments("--module-path", modulePath) //
 				.addArguments("--module", "org.junit.platform.console") //
 				.addArguments("--version", "--disable-ansi-colors") //
 				.putEnvironment("CLICOLOR_FORCE", "1") // enable ANSI colors by default (see https://picocli.info/#_heuristics_for_enabling_ansi)
-				.build() //
-				.run();
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(0, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(0, result.exitCode());
 
 		var version = Helper.version("junit-platform-console");
 		assertLinesMatch("""
@@ -140,49 +148,38 @@ class StandaloneTests {
 				JVM: .*
 				OS: .*
 				""".formatted(version).lines(), //
-			result.getOutputLines("out").stream());
+			result.stdOut().lines());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(1)
-	void compile() throws Exception {
-		var workspace = Request.WORKSPACE.resolve(Projects.STANDALONE);
-		var result = Request.builder() //
-				.setTool(new Javac()) //
-				.setProject(Projects.STANDALONE) //
+	@Execution(SAME_THREAD)
+	void compile(@FilePrefix("javac") OutputFiles javacOutputFiles) throws Exception {
+		var result = ProcessStarters.javaCommand("javac") //
+				.workingDir(workspace) //
 				.addArguments("-Xlint:-options") //
 				.addArguments("--release", "8") //
 				.addArguments("-proc:none") //
-				.addArguments("-d", workspace.resolve("bin")) //
-				.addArguments("--class-path", MavenRepo.jar("junit-platform-console-standalone")) //
-				.addArguments(workspace.resolve("src/standalone/JupiterIntegration.java")) //
-				.addArguments(workspace.resolve("src/standalone/JupiterParamsIntegration.java")) //
-				.addArguments(workspace.resolve("src/standalone/SuiteIntegration.java")) //
-				.addArguments(workspace.resolve("src/standalone/VintageIntegration.java")).build() //
-				.run();
+				.addArguments("-d", workspace.resolve("bin").toString()) //
+				.addArguments("--class-path", MavenRepo.jar("junit-platform-console-standalone").toString()) //
+				.addArguments(workspace.resolve("src/other/OtherwiseNotReferencedClass.java").toString()) //
+				.addArguments(workspace.resolve("src/standalone/JupiterIntegration.java").toString()) //
+				.addArguments(workspace.resolve("src/standalone/JupiterParamsIntegration.java").toString()) //
+				.addArguments(workspace.resolve("src/standalone/SuiteIntegration.java").toString()) //
+				.addArguments(workspace.resolve("src/standalone/VintageIntegration.java").toString()) //
+				.redirectOutput(javacOutputFiles) //
+				.startAndWait();
 
-		assertEquals(0, result.getExitCode(), () -> getExitCodeMessage(result));
-		assertTrue(result.getOutput("out").isEmpty());
-		assertTrue(result.getOutput("err").isEmpty());
-
-		// create "tests.jar" that'll be picked-up by "testWithJarredTestClasses()" later
-		var jarFolder = Files.createDirectories(workspace.resolve("jar"));
-		var jarResult = Request.builder() //
-				.setTool(new Jar()) //
-				.setProject(Projects.STANDALONE) //
-				.addArguments("--create") //
-				.addArguments("--file", jarFolder.resolve("tests.jar")) //
-				.addArguments("-C", workspace.resolve("bin"), ".") //
-				.build().run(false);
-		assertEquals(0, jarResult.getExitCode(), String.join("\n", jarResult.getOutputLines("out")));
+		assertEquals(0, result.exitCode());
+		assertTrue(result.stdOut().isEmpty());
+		assertTrue(result.stdErr().isEmpty());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(2)
-	void discoverTree() {
-		Result result = discover("--details-theme=ascii");
+	@Execution(SAME_THREAD)
+	void discoverTree(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = discover(outputFiles, "--details-theme=ascii");
 
 		var expected = """
 				.
@@ -211,14 +208,14 @@ class StandaloneTests {
 				[         9 tests found      ]
 
 				""".stripIndent();
-		assertLinesMatch(expected.lines(), result.getOutputLines("out").stream());
+		assertLinesMatch(expected.lines(), result.stdOut().lines());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(2)
-	void discoverFlat() {
-		Result result = discover("--details=flat");
+	@Execution(SAME_THREAD)
+	void discoverFlat(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = discover(outputFiles, "--details=flat");
 
 		var expected = """
 				JUnit Platform Suite ([engine:junit-platform-suite])
@@ -246,13 +243,14 @@ class StandaloneTests {
 				[         9 tests found      ]
 
 				""".stripIndent();
-		assertLinesMatch(expected.lines(), result.getOutputLines("out").stream());
+		assertLinesMatch(expected.lines(), result.stdOut().lines());
 	}
 
 	@Test
 	@Order(2)
-	void discoverVerbose() {
-		Result result = discover("--details=verbose", "--details-theme=ascii");
+	@Execution(SAME_THREAD)
+	void discoverVerbose(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = discover(outputFiles, "--details=verbose", "--details-theme=ascii");
 
 		var expected = """
 				+-- JUnit Platform Suite
@@ -330,23 +328,23 @@ class StandaloneTests {
 				[         9 tests found      ]
 
 				""".stripIndent();
-		assertLinesMatch(expected.lines(), result.getOutputLines("out").stream());
+		assertLinesMatch(expected.lines(), result.stdOut().lines());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(2)
-	void discoverNone() {
-		Result result = discover("--details=none");
+	@Execution(SAME_THREAD)
+	void discoverNone(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = discover(outputFiles, "--details=none");
 
-		assertThat(result.getOutputLines("out")).isEmpty();
+		assertThat(result.stdOut()).isEmpty();
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(2)
-	void discoverSummary() {
-		Result result = discover("--details=summary");
+	@Execution(SAME_THREAD)
+	void discoverSummary(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = discover(outputFiles, "--details=summary");
 
 		var expected = """
 
@@ -354,14 +352,14 @@ class StandaloneTests {
 				[         9 tests found      ]
 
 				""".stripIndent();
-		assertLinesMatch(expected.lines(), result.getOutputLines("out").stream());
+		assertLinesMatch(expected.lines(), result.stdOut().lines());
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(2)
-	void discoverTestFeed() {
-		Result result = discover("--details=testfeed");
+	@Execution(SAME_THREAD)
+	void discoverTestFeed(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = discover(outputFiles, "--details=testfeed");
 		var expected = """
 				JUnit Platform Suite > SuiteIntegration > JUnit Jupiter > SuiteIntegration$SingleTestContainer > successful()
 				JUnit Jupiter > JupiterIntegration > successful()
@@ -378,142 +376,120 @@ class StandaloneTests {
 
 				""".stripIndent();
 
-		assertLinesMatch(expected.lines(), result.getOutputLines("out").stream());
+		assertLinesMatch(expected.lines(), result.stdOut().lines());
 	}
 
-	private static Result discover(String... args) {
-		var result = Request.builder() //
+	private static ProcessResult discover(OutputFiles outputFiles, String... args) throws Exception {
+		var result = ProcessStarters.java() //
+				.workingDir(workspace) //
 				.putEnvironment("NO_COLOR", "1") // --disable-ansi-colors
-				.setTool(new Java()) //
-				.setProject(Projects.STANDALONE) //
-				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone")) //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
 				.addArguments("discover") //
 				.addArguments("--scan-class-path") //
 				.addArguments("--disable-banner") //
 				.addArguments("--include-classname", "standalone.*") //
 				.addArguments("--classpath", "bin") //
-				.addArguments((Object[]) args) //
-				.build() //
-				.run(false);
+				.addArguments(args) //
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(0, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(0, result.exitCode());
 		return result;
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(3)
-	void execute() throws IOException {
-		var result = Request.builder() //
+	@Execution(SAME_THREAD)
+	void execute(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var result = ProcessStarters.java() //
+				.workingDir(workspace) //
 				.putEnvironment("NO_COLOR", "1") // --disable-ansi-colors
-				.setTool(new Java()) //
-				.setProject(Projects.STANDALONE) //
 				.addArguments("--show-version") //
 				.addArguments("-enableassertions") //
 				.addArguments("-Djava.util.logging.config.file=logging.properties") //
 				.addArguments("-Djunit.platform.launcher.interceptors.enabled=true") //
-				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone")) //
+				.addArguments("-Duser.language=en", "-Duser.country=US") //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
 				.addArguments("execute") //
 				.addArguments("--scan-class-path") //
 				.addArguments("--disable-banner") //
 				.addArguments("--include-classname", "standalone.*") //
-				.addArguments("--classpath", "bin").build() //
-				.run(false);
+				.addArguments("--classpath", "bin") //
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(1, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(1, result.exitCode());
 
-		var workspace = Request.WORKSPACE.resolve(Projects.STANDALONE);
-		var expectedOutLines = Files.readAllLines(workspace.resolve("expected-out.txt"));
-		var expectedErrLines = Files.readAllLines(workspace.resolve("expected-err.txt"));
-		assertLinesMatch(expectedOutLines, result.getOutputLines("out"));
-		List<String> actualErrLines = result.getOutputLines("err");
-		if (actualErrLines.getFirst().contains("stty: /dev/tty: No such device or address")) {
-			// Happens intermittently on GitHub Actions on Windows
-			actualErrLines = new ArrayList<>(actualErrLines);
-			actualErrLines.removeFirst();
-		}
-		assertLinesMatch(expectedErrLines, actualErrLines);
-
-		var jupiterVersion = Helper.version("junit-jupiter-engine");
-		var vintageVersion = Helper.version("junit-vintage-engine");
-		assertTrue(result.getOutput("err").contains("junit-jupiter"
-				+ " (group ID: org.junit.jupiter, artifact ID: junit-jupiter-engine, version: " + jupiterVersion));
-		assertTrue(result.getOutput("err").contains("junit-vintage"
-				+ " (group ID: org.junit.vintage, artifact ID: junit-vintage-engine, version: " + vintageVersion));
+		assertOutputOnCurrentJvm(result);
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(4)
-	void executeOnJava8() throws IOException {
-		Java java8 = getJava8();
-		var result = Request.builder() //
-				.setTool(java8) //
-				.setJavaHome(java8.getHome()) //
-				.setProject(Projects.STANDALONE) //
+	@Execution(SAME_THREAD)
+	void executeOnJava8(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var java8Home = Helper.getJavaHome(8).orElseThrow(TestAbortedException::new);
+		var result = ProcessStarters.java(java8Home) //
+				.workingDir(workspace) //
 				.addArguments("-showversion") //
 				.addArguments("-enableassertions") //
 				.addArguments("-Djava.util.logging.config.file=logging.properties") //
 				.addArguments("-Djunit.platform.launcher.interceptors.enabled=true") //
-				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone")) //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
 				.addArguments("execute") //
 				.addArguments("--scan-class-path") //
 				.addArguments("--disable-banner") //
 				.addArguments("--include-classname", "standalone.*") //
-				.addArguments("--classpath", "bin").build() //
-				.run(false);
+				.addArguments("--classpath", "bin") //
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(1, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(1, result.exitCode());
 
-		var workspace = Request.WORKSPACE.resolve(Projects.STANDALONE);
 		var expectedOutLines = Files.readAllLines(workspace.resolve("expected-out.txt"));
 		var expectedErrLines = getExpectedErrLinesOnJava8(workspace);
-		assertLinesMatch(expectedOutLines, result.getOutputLines("out"));
-		assertLinesMatch(expectedErrLines, result.getOutputLines("err"));
+		assertLinesMatch(expectedOutLines, result.stdOutLines());
+		assertLinesMatch(expectedErrLines, result.stdErrLines());
 
 		var jupiterVersion = Helper.version("junit-jupiter-engine");
 		var vintageVersion = Helper.version("junit-vintage-engine");
-		assertTrue(result.getOutput("err").contains("junit-jupiter"
+		assertTrue(result.stdErr().contains("junit-jupiter"
 				+ " (group ID: org.junit.jupiter, artifact ID: junit-jupiter-engine, version: " + jupiterVersion));
-		assertTrue(result.getOutput("err").contains("junit-vintage"
+		assertTrue(result.stdErr().contains("junit-vintage"
 				+ " (group ID: org.junit.vintage, artifact ID: junit-vintage-engine, version: " + vintageVersion));
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(5)
-	// https://github.com/junit-team/junit5/issues/2600
-	void executeOnJava8SelectPackage() throws IOException {
-		Java java8 = getJava8();
-		var result = Request.builder() //
-				.setTool(java8) //
-				.setJavaHome(java8.getHome()) //
-				.setProject(Projects.STANDALONE) //
-				.addArguments("-showversion") //
+	@Execution(SAME_THREAD)
+	// https://github.com/junit-team/junit-framework/issues/2600
+	void executeOnJava8SelectPackage(@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var java8Home = Helper.getJavaHome(8).orElseThrow(TestAbortedException::new);
+		var result = ProcessStarters.java(java8Home) //
+				.workingDir(workspace).addArguments("-showversion") //
 				.addArguments("-enableassertions") //
 				.addArguments("-Djava.util.logging.config.file=logging.properties") //
 				.addArguments("-Djunit.platform.launcher.interceptors.enabled=true") //
-				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone")) //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
 				.addArguments("execute") //
 				.addArguments("--select-package", Projects.STANDALONE) //
 				.addArguments("--disable-banner") //
 				.addArguments("--include-classname", "standalone.*") //
-				.addArguments("--classpath", "bin").build() //
-				.run(false);
+				.addArguments("--classpath", "bin") //
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(1, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(1, result.exitCode());
 
-		var workspace = Request.WORKSPACE.resolve(Projects.STANDALONE);
 		var expectedOutLines = Files.readAllLines(workspace.resolve("expected-out.txt"));
 		var expectedErrLines = getExpectedErrLinesOnJava8(workspace);
-		assertLinesMatch(expectedOutLines, result.getOutputLines("out"));
-		assertLinesMatch(expectedErrLines, result.getOutputLines("err"));
+		assertLinesMatch(expectedOutLines, result.stdOutLines());
+		assertLinesMatch(expectedErrLines, result.stdErrLines());
 
 		var jupiterVersion = Helper.version("junit-jupiter-engine");
 		var vintageVersion = Helper.version("junit-vintage-engine");
-		assertTrue(result.getOutput("err").contains("junit-jupiter"
+		assertTrue(result.stdErr().contains("junit-jupiter"
 				+ " (group ID: org.junit.jupiter, artifact ID: junit-jupiter-engine, version: " + jupiterVersion));
-		assertTrue(result.getOutput("err").contains("junit-vintage"
+		assertTrue(result.stdErr().contains("junit-vintage"
 				+ " (group ID: org.junit.vintage, artifact ID: junit-vintage-engine, version: " + vintageVersion));
 	}
 
@@ -524,55 +500,61 @@ class StandaloneTests {
 		return expectedErrLines;
 	}
 
-	@ResourceLock(Projects.STANDALONE)
 	@Test
 	@Order(6)
-	@Disabled("https://github.com/junit-team/junit5/issues/1724")
-	void executeWithJarredTestClasses() {
-		var jar = MavenRepo.jar("junit-platform-console-standalone");
-		var path = new ArrayList<String>();
-		// path.add("bin"); // "exploded" test classes are found, see also test() above
-		path.add(Request.WORKSPACE.resolve("standalone/jar/tests.jar").toAbsolutePath().toString());
-		path.add(jar.toString());
-		var result = Request.builder() //
-				.setTool(new Java()) //
-				.setProject(Projects.STANDALONE) //
+	@Execution(SAME_THREAD)
+	void executeWithJarredTestClasses(@FilePrefix("jar") OutputFiles jarOutputFiles,
+			@FilePrefix("console-launcher") OutputFiles outputFiles) throws Exception {
+		var jar = workspace.resolve("tests.jar");
+		var jarResult = ProcessStarters.javaCommand("jar") //
+				.workingDir(workspace) //
+				.addArguments("--create") //
+				.addArguments("--file", jar.toAbsolutePath().toString()) //
+				.addArguments("-C", workspace.resolve("bin").toString(), ".") //
+				.redirectOutput(jarOutputFiles) //
+				.startAndWait();
+
+		assertEquals(0, jarResult.exitCode());
+
+		var result = ProcessStarters.java() //
+				.workingDir(workspace) //
+				.putEnvironment("NO_COLOR", "1") // --disable-ansi-colors
 				.addArguments("--show-version") //
 				.addArguments("-enableassertions") //
 				.addArguments("-Djava.util.logging.config.file=logging.properties") //
-				.addArguments("--class-path", String.join(File.pathSeparator, path)) //
-				.addArguments("org.junit.platform.console.ConsoleLauncher") //
+				.addArguments("-Djunit.platform.launcher.interceptors.enabled=true") //
+				.addArguments("-Duser.language=en", "-Duser.country=US") //
+				.addArguments("-jar", MavenRepo.jar("junit-platform-console-standalone").toString()) //
 				.addArguments("execute") //
 				.addArguments("--scan-class-path") //
 				.addArguments("--disable-banner") //
 				.addArguments("--include-classname", "standalone.*") //
-				.addArguments("--fail-if-no-tests") //
-				.build() //
-				.run(false);
+				.addArguments("--classpath", jar.toAbsolutePath().toString()) //
+				.redirectOutput(outputFiles) //
+				.startAndWait();
 
-		assertEquals(1, result.getExitCode(), () -> getExitCodeMessage(result));
+		assertEquals(1, result.exitCode());
+
+		assertOutputOnCurrentJvm(result);
 	}
 
-	private static String getExitCodeMessage(Result result) {
-		return "Exit codes don't match. Stdout:\n" + result.getOutput("out") + //
-				"\n\nStderr:\n" + result.getOutput("err") + "\n";
-	}
+	private static void assertOutputOnCurrentJvm(ProcessResult result) throws IOException {
+		var expectedOutLines = Files.readAllLines(workspace.resolve("expected-out.txt"));
+		var expectedErrLines = Files.readAllLines(workspace.resolve("expected-err.txt"));
+		assertLinesMatch(expectedOutLines, result.stdOutLines());
+		var actualErrLines = result.stdErrLines();
+		if (actualErrLines.getFirst().contains("stty: /dev/tty: No such device or address")) {
+			// Happens intermittently on GitHub Actions on Windows
+			actualErrLines = new ArrayList<>(actualErrLines);
+			actualErrLines.removeFirst();
+		}
+		assertLinesMatch(expectedErrLines, actualErrLines);
 
-	/**
-	 * Special override of class {@link Java} to resolve against a different {@code JAVA_HOME}.
-	 */
-	private static Java getJava8() {
-		Path java8Home = Helper.getJavaHome("8").orElseThrow(TestAbortedException::new);
-		return new Java() {
-			@Override
-			public Path getHome() {
-				return java8Home;
-			}
-
-			@Override
-			public String getVersion() {
-				return "8";
-			}
-		};
+		var jupiterVersion = Helper.version("junit-jupiter-engine");
+		var vintageVersion = Helper.version("junit-vintage-engine");
+		assertTrue(result.stdErr().contains("junit-jupiter"
+				+ " (group ID: org.junit.jupiter, artifact ID: junit-jupiter-engine, version: " + jupiterVersion));
+		assertTrue(result.stdErr().contains("junit-vintage"
+				+ " (group ID: org.junit.vintage, artifact ID: junit-vintage-engine, version: " + vintageVersion));
 	}
 }
