@@ -1,5 +1,5 @@
 /*
- * Copyright 2021-2022 the original author or authors.
+ * Copyright 2021-2025 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,18 +16,27 @@
 
 package org.opentest4j.reporting.events.core;
 
+import org.apiguardian.api.API;
 import org.opentest4j.reporting.events.api.ChildElement;
 import org.opentest4j.reporting.events.api.Context;
 import org.opentest4j.reporting.events.root.Event;
 import org.opentest4j.reporting.schema.Namespace;
 import org.opentest4j.reporting.schema.QualifiedName;
 
+import static org.apiguardian.api.API.Status.EXPERIMENTAL;
+
 /**
  * The {@code metadata} element of the core namespace.
+ *
+ * @since 0.1.0
  */
+@API(status = EXPERIMENTAL, since = "0.1.0")
 public class Metadata extends ChildElement<Event, Metadata> {
 
-	private static final QualifiedName ELEMENT = QualifiedName.of(Namespace.REPORTING_CORE, "metadata");
+	/**
+	 * Qualified name of the {@code metadata} element.
+	 */
+	public static final QualifiedName ELEMENT = QualifiedName.of(Namespace.REPORTING_CORE, "metadata");
 
 	Metadata(Context context) {
 		super(context, ELEMENT);
