@@ -1,5 +1,5 @@
 /*
- * Copyright 2021-2022 the original author or authors.
+ * Copyright 2021-2025 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.opentest4j.reporting.events.core;
 
+import org.apiguardian.api.API;
 import org.opentest4j.reporting.events.api.ChildElement;
 import org.opentest4j.reporting.events.api.Context;
 import org.opentest4j.reporting.schema.Namespace;
@@ -23,9 +24,14 @@ import org.opentest4j.reporting.schema.QualifiedName;
 
 import java.net.URI;
 
+import static org.apiguardian.api.API.Status.EXPERIMENTAL;
+
 /**
  * The {@code uriSource} element of the core namespace.
+ *
+ * @since 0.1.0
  */
+@API(status = EXPERIMENTAL, since = "0.1.0")
 public class UriSource extends ChildElement<Sources, UriSource> {
 
 	private static final QualifiedName ELEMENT = QualifiedName.of(Namespace.REPORTING_CORE, "uriSource");
