@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2025, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2026, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,9 +33,11 @@ import java.io.FileNotFoundException;
 import java.io.FileReader;
 import java.io.FileWriter;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.StringReader;
+import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.nio.file.Files;
@@ -377,10 +379,18 @@ public class CompileAction extends Action {
             // run jasm and jcod first (if needed) in case the resulting class
             // files will be required when compiling the .java files.
             status = passed("Not yet run");
-            if (status.isPassed() && !jasmArgs.isEmpty())
+            if (status.isPassed() && !jasmArgs.isEmpty()) {
+                if (!jasmArgs.contains("-cv")) {
+                    // use 45.3 (representing Java 1.1) as the default
+                    // class file version, if the jasm file(s) don't
+                    // have any specific class file version in them
+                    jasmArgs.addAll(List.of("-cv", "45.3"));
+                }
                 status = jasm(jasmArgs);
-            if (status.isPassed() && !jcodArgs.isEmpty())
+            }
+            if (status.isPassed() && !jcodArgs.isEmpty()) {
                 status = jcod(jcodArgs);
+            }
             if (status.isPassed() && runJavac) {
                 javacArgs = getJavacCommandArgs(javacArgs);
                 for (String arg: javacArgs) {
@@ -414,31 +424,36 @@ public class CompileAction extends Action {
 
     //----------internal methods------------------------------------------------
 
-    private Status jasm(List<String> files) {
-        return asmtools("jasm", files);
+    private Status jasm(List<String> jasmArgs) {
+        return asmtools("jasm", jasmArgs);
     }
 
-    private Status jcod(List<String> files) {
-        return asmtools("jcoder", files);
+    private Status jcod(List<String> jcodArgs) {
+        return asmtools("jcoder", jcodArgs);
     }
 
-    private Status asmtools(String toolName, List<String> files) {
-        if (files.isEmpty())
+    private Status asmtools(String toolName, List<String> args) {
+        if (args.isEmpty())
             return Status.passed(toolName + ": no files");
 
         List<String> toolArgs = new ArrayList<>();
         toolArgs.add("-d");
         toolArgs.add(destDir.getPath());
-        toolArgs.addAll(files);
+        toolArgs.addAll(args);
         try {
             String toolClassName = "org.openjdk.asmtools." + toolName + ".Main";
             recorder.asmtools(toolClassName, toolArgs);
             Class<?> toolClass = Class.forName(toolClassName);
-            Constructor<?> constr = toolClass.getConstructor(PrintStream.class, String.class);
-            ByteArrayOutputStream baos = new ByteArrayOutputStream();
-            PrintStream ps = new PrintStream(baos);
-            try {
-                Object tool = constr.newInstance(ps, toolName);
+            Constructor<?> constr = getAsmToolConstructor(toolClass);
+            ByteArrayOutputStream toolOutput = new ByteArrayOutputStream();
+            ByteArrayOutputStream toolStdout = new ByteArrayOutputStream();
+            ByteArrayOutputStream toolStderr = new ByteArrayOutputStream();
+
+            try (var output = new PrintStream(toolOutput);
+                 var stdout = new PrintStream(toolStdout);
+                 var stderr = new PrintStream(toolStderr)) {
+
+                Object tool = createAsmToolInstance(constr, output, stdout, stderr);
                 Method m = toolClass.getMethod("compile", String[].class);
                 Object r = m.invoke(tool, new Object[] { toolArgs.toArray(new String[0]) });
                 if (r instanceof Boolean) {
@@ -447,8 +462,16 @@ public class CompileAction extends Action {
                 } else
                     return Status.error("unexpected result from " + toolName + ": " + r.toString());
             } finally {
+                // print out the stdout and stderr content generated by the tool
+                try (PrintWriter out = section.createOutput(toolName + "-stdout")) {
+                    out.write(toolStdout.toString());
+                }
+                try (PrintWriter out = section.createOutput(toolName + "-stderr")) {
+                    out.write(toolStderr.toString());
+                }
+                // print out the tool's output
                 try (PrintWriter out = section.createOutput(toolName)) {
-                    out.write(baos.toString());
+                    out.write(toolOutput.toString());
                 }
             }
         } catch (ClassNotFoundException e) {
@@ -458,6 +481,36 @@ public class CompileAction extends Action {
         }
     }
 
+    private static <T> Constructor<T> getAsmToolConstructor(final Class<T> toolMainClass)
+            throws ReflectiveOperationException {
+        final Class<?> toolOutputClass = Class.forName("org.openjdk.asmtools.common.outputs.ToolOutput");
+        final Class<?> logClass = Class.forName("org.openjdk.asmtools.common.outputs.log.DualStreamToolOutput");
+        final Class<?> toolInputClassArr = Class.forName("[Lorg.openjdk.asmtools.common.inputs.ToolInput;");
+        // Constructor of the form Klass(ToolOutput toolOutput, DualStreamToolOutput log, ToolInput... toolInputs)
+        return toolMainClass.getConstructor(toolOutputClass, logClass, toolInputClassArr);
+    }
+
+    private static Object createAsmToolInstance(final Constructor<?> constructor,
+                                                final PrintStream toolOutput,
+                                                final PrintStream toolStdout,
+                                                final PrintStream toolStderr)
+            throws ReflectiveOperationException {
+        final Object printWriterOutput =
+                Class.forName("org.openjdk.asmtools.common.outputs.PrintWriterOutput")
+                        .getConstructor(OutputStream.class)
+                        .newInstance(toolOutput);
+        final Object stdOutErrOutput =
+                Class.forName("org.openjdk.asmtools.common.outputs.log.DualOutputStreamOutput")
+                        .getConstructor(PrintStream.class, PrintStream.class)
+                        .newInstance(toolStdout, toolStderr);
+        final Class<?> toolInputClass = Class.forName("org.openjdk.asmtools.common.inputs.ToolInput");
+        // construct the instance in the form of:
+        //  new Klass(new PrintWriterOutput(toolOutput),
+        //           new DualOutputStreamOutput(toolStdout, toolStderr),
+        //           new ToolInput[0]);
+        return constructor.newInstance(printWriterOutput, stdOutErrOutput, Array.newInstance(toolInputClass, 0));
+    }
+
     /**
      * Determine the arguments for the compilation.
      * Three different types of compilation are supported.
